# Porting an AMS sensor driver and operation in the Linux kernel.
version 1.1



## Description

  - `arch/arm/boot/dts/`
    This contains the device tree overlay file that is an example of how the
    sensor was added to a Raspberry Pi BSP for development/testing.

  - `driver/`
    This directory contains the source and includes to the device driver and
    was built and tested against linux kernel 4.9 on a Raspberry Pi 3B

  - `include/`
    This directory contains the device specific include files that are needed
    by both the board config source and the driver source.



## Cross-Compiling

  Follow these steps, then the subsection for the component you are compiling -

  1. Download Raspberry Pi Cross Compile Toolchain:
     ```
     git clone git://github.com/raspberrypi/tools.git --depth=1
     ```

  2. Set `RPI_TOOLCHAIN` environment variable to point to the toolchain downloaded in Step 1.

  3. Download Raspberry Pi Linux Kernel source:
     1. `git clone --branch <kernel_version_branch> --depth=1 https://github.com/raspberrypi/linux`
     2. Driver tested against **Kernel version 4.9.59-v7+**, tag=**raspberrypi-kernel_1.20171029-1**

  4. Set `LINUX_SRC` environment variable to point to the kernel source downloaded in Step 3.

  5. A convenience script `./rpi.env` is included to setup Cross Compilation environment
     variables before compiling
     `source ./rpi.env`   **requires RPI_TOOLCHAIN and LINUX_SRC env variables set**


  6. Compile Raspberry Pi Linux Kernel
     1. `make bcm2709_defconfig`
     2. `make modules_prepare`
     3. `make -j 4 zImage modules dtbs`
        This can take a while, as it is compiling the whole kernel. This
        step is necessary to create the Module.symvers needed to build
        external modules
     4. Reference Material for Raspberry Pi Cross compiling:
        - [Official Raspberry Pi documentation for building the kernel][2]
        - [Other Raspberry Pi Kernel compilation resources][1]

  7. Compile Device Driver:
     1. `source ./rpi.env` to setup for cross compiling
     2. `cd ./driver`
     3. `make clean`
     4. `make CONFIG_SENSORS_TCS3408=m`
     5. copy tcs3408.ko to Raspberry Pi

  8. Compile Device Tree overlay
     1. `cd ./arch/arm/boot/dts`
     2. `dtc -@ -I dts -O dtb -o tcs3408-overlay.dtbo tcs3408-overlay.dts`
     3. `cp tcs3408-overlay.dtbo  /boot/overlays/`  (on the Raspberry Pi)
     4. add `dtoverlay=tcs3408-overlay` to end of text file `/boot/config.txt`
     5. Raspberry Pi must be rebooted to take effect



## Operation Guide

  1. In the Raspberry Pi shell, become root with:
     ```
     sudo su
     ```

  2. Transfer the driver module to the raspberry pi (if it is not already in the image).

  3. Inserting/Removing the driver module
     ```
     insmod /home/pi/example.ko
     rmmod example
     ```


## Hardware Wiring to Raspberry Pi 3b

  This diagram is for wiring an Official AMS 7-pin evaluation Daughter card.

  [Raspberry Pi 3b 40-pin connector reference][3]


    Raspberry Pi 3b:                                 AMS 7-pin Daughter card:

                    -----                                              --
    Pin_1    3.3V - |* *|                                         X --| Pin_1
          I2C_SDA - |* *|                                       VDD --|
          I2C_SCL - |* *| - GND                                 GND --|
                    |* *|                                   I2C_SDA --|
                    |* *|                                   I2C_SCL --|
                    |* *|                                       INT --|
                    |* *|                                         X --|
                    |* *|                                              --
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *| - GPIO20 (INT)
                    |* *|
                    -----

[1]: http://elinux.org/Raspberry_Pi_Kernel_Compilation
[2]: https://www.raspberrypi.org/documentation/linux/kernel/building.md
[3]: https://pinout.xyz/
