/*
 *****************************************************************************
 * Copyright by ams AG                                                       *
 * All rights are reserved.                                                  *
 *                                                                           *
 * IMPORTANT - PLEASE READ CAREFULLY BEFORE COPYING, INSTALLING OR USING     *
 * THE SOFTWARE.                                                             *
 *                                                                           *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         *
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE     *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.      *
 *****************************************************************************
 */

#include <linux/slab.h>
#include "ams_fft.h"

// Pseudo code for generation of the Twiddle Coefficients...
// for (i=0; i<(3*FFT_N/4); i++)
// {
//     twiddleCoef[2*i] = cos(i * 2 * PI / (double)FFT_N);
//     twiddleCoef[2*i+1] = sin(i * 2 * PI / (double)FFT_N);
// }
// Twiddle Coefficients for 16-bit 1024-bin FFT
static const uint16_t twiddles[1536] = {
    0x7fff, 0x0000, 0x7ffe, 0x00c9, 0x7ffc, 0x0192, 0x7ff9, 0x025b,
    0x7ff5, 0x0324, 0x7fef, 0x03ed, 0x7fe8, 0x04b6, 0x7fe0, 0x057e,
    0x7fd7, 0x0647, 0x7fcd, 0x0710, 0x7fc1, 0x07d9, 0x7fb4, 0x08a1,
    0x7fa6, 0x096a, 0x7f96, 0x0a32, 0x7f86, 0x0afb, 0x7f74, 0x0bc3,
    0x7f61, 0x0c8b, 0x7f4c, 0x0d53, 0x7f37, 0x0e1b, 0x7f20, 0x0ee3,
    0x7f08, 0x0fab, 0x7eef, 0x1072, 0x7ed4, 0x1139, 0x7eb9, 0x1200,
    0x7e9c, 0x12c7, 0x7e7e, 0x138e, 0x7e5e, 0x1455, 0x7e3e, 0x151b,
    0x7e1c, 0x15e1, 0x7df9, 0x16a7, 0x7dd5, 0x176d, 0x7db0, 0x1833,
    0x7d89, 0x18f8, 0x7d61, 0x19bd, 0x7d38, 0x1a82, 0x7d0e, 0x1b46,
    0x7ce2, 0x1c0b, 0x7cb6, 0x1ccf, 0x7c88, 0x1d93, 0x7c59, 0x1e56,
    0x7c29, 0x1f19, 0x7bf7, 0x1fdc, 0x7bc4, 0x209f, 0x7b91, 0x2161,
    0x7b5c, 0x2223, 0x7b25, 0x22e4, 0x7aee, 0x23a6, 0x7ab5, 0x2467,
    0x7a7c, 0x2527, 0x7a41, 0x25e7, 0x7a04, 0x26a7, 0x79c7, 0x2767,
    0x7989, 0x2826, 0x7949, 0x28e5, 0x7908, 0x29a3, 0x78c6, 0x2a61,
    0x7883, 0x2b1e, 0x783f, 0x2bdb, 0x77f9, 0x2c98, 0x77b3, 0x2d54,
    0x776b, 0x2e10, 0x7722, 0x2ecc, 0x76d8, 0x2f86, 0x768d, 0x3041,
    0x7640, 0x30fb, 0x75f3, 0x31b4, 0x75a4, 0x326d, 0x7554, 0x3326,
    0x7503, 0x33de, 0x74b1, 0x3496, 0x745e, 0x354d, 0x740a, 0x3603,
    0x73b5, 0x36b9, 0x735e, 0x376f, 0x7306, 0x3824, 0x72ae, 0x38d8,
    0x7254, 0x398c, 0x71f9, 0x3a3f, 0x719d, 0x3af2, 0x7140, 0x3ba4,
    0x70e1, 0x3c56, 0x7082, 0x3d07, 0x7022, 0x3db7, 0x6fc0, 0x3e67,
    0x6f5e, 0x3f16, 0x6efa, 0x3fc5, 0x6e95, 0x4073, 0x6e30, 0x4120,
    0x6dc9, 0x41cd, 0x6d61, 0x4279, 0x6cf8, 0x4325, 0x6c8e, 0x43d0,
    0x6c23, 0x447a, 0x6bb7, 0x4523, 0x6b4a, 0x45cc, 0x6adb, 0x4674,
    0x6a6c, 0x471c, 0x69fc, 0x47c3, 0x698b, 0x4869, 0x6919, 0x490e,
    0x68a5, 0x49b3, 0x6831, 0x4a57, 0x67bc, 0x4afa, 0x6745, 0x4b9d,
    0x66ce, 0x4c3f, 0x6656, 0x4ce0, 0x65dd, 0x4d80, 0x6562, 0x4e20,
    0x64e7, 0x4ebf, 0x646b, 0x4f5d, 0x63ee, 0x4ffa, 0x6370, 0x5097,
    0x62f1, 0x5133, 0x6271, 0x51ce, 0x61f0, 0x5268, 0x616e, 0x5301,
    0x60eb, 0x539a, 0x6067, 0x5432, 0x5fe2, 0x54c9, 0x5f5d, 0x555f,
    0x5ed6, 0x55f4, 0x5e4f, 0x5689, 0x5dc6, 0x571d, 0x5d3d, 0x57b0,
    0x5cb3, 0x5842, 0x5c28, 0x58d3, 0x5b9c, 0x5963, 0x5b0f, 0x59f3,
    0x5a81, 0x5a81, 0x59f3, 0x5b0f, 0x5963, 0x5b9c, 0x58d3, 0x5c28,
    0x5842, 0x5cb3, 0x57b0, 0x5d3d, 0x571d, 0x5dc6, 0x5689, 0x5e4f,
    0x55f4, 0x5ed6, 0x555f, 0x5f5d, 0x54c9, 0x5fe2, 0x5432, 0x6067,
    0x539a, 0x60eb, 0x5301, 0x616e, 0x5268, 0x61f0, 0x51ce, 0x6271,
    0x5133, 0x62f1, 0x5097, 0x6370, 0x4ffa, 0x63ee, 0x4f5d, 0x646b,
    0x4ebf, 0x64e7, 0x4e20, 0x6562, 0x4d80, 0x65dd, 0x4ce0, 0x6656,
    0x4c3f, 0x66ce, 0x4b9d, 0x6745, 0x4afa, 0x67bc, 0x4a57, 0x6831,
    0x49b3, 0x68a5, 0x490e, 0x6919, 0x4869, 0x698b, 0x47c3, 0x69fc,
    0x471c, 0x6a6c, 0x4674, 0x6adb, 0x45cc, 0x6b4a, 0x4523, 0x6bb7,
    0x447a, 0x6c23, 0x43d0, 0x6c8e, 0x4325, 0x6cf8, 0x4279, 0x6d61,
    0x41cd, 0x6dc9, 0x4120, 0x6e30, 0x4073, 0x6e95, 0x3fc5, 0x6efa,
    0x3f16, 0x6f5e, 0x3e67, 0x6fc0, 0x3db7, 0x7022, 0x3d07, 0x7082,
    0x3c56, 0x70e1, 0x3ba4, 0x7140, 0x3af2, 0x719d, 0x3a3f, 0x71f9,
    0x398c, 0x7254, 0x38d8, 0x72ae, 0x3824, 0x7306, 0x376f, 0x735e,
    0x36b9, 0x73b5, 0x3603, 0x740a, 0x354d, 0x745e, 0x3496, 0x74b1,
    0x33de, 0x7503, 0x3326, 0x7554, 0x326d, 0x75a4, 0x31b4, 0x75f3,
    0x30fb, 0x7640, 0x3041, 0x768d, 0x2f86, 0x76d8, 0x2ecc, 0x7722,
    0x2e10, 0x776b, 0x2d54, 0x77b3, 0x2c98, 0x77f9, 0x2bdb, 0x783f,
    0x2b1e, 0x7883, 0x2a61, 0x78c6, 0x29a3, 0x7908, 0x28e5, 0x7949,
    0x2826, 0x7989, 0x2767, 0x79c7, 0x26a7, 0x7a04, 0x25e7, 0x7a41,
    0x2527, 0x7a7c, 0x2467, 0x7ab5, 0x23a6, 0x7aee, 0x22e4, 0x7b25,
    0x2223, 0x7b5c, 0x2161, 0x7b91, 0x209f, 0x7bc4, 0x1fdc, 0x7bf7,
    0x1f19, 0x7c29, 0x1e56, 0x7c59, 0x1d93, 0x7c88, 0x1ccf, 0x7cb6,
    0x1c0b, 0x7ce2, 0x1b46, 0x7d0e, 0x1a82, 0x7d38, 0x19bd, 0x7d61,
    0x18f8, 0x7d89, 0x1833, 0x7db0, 0x176d, 0x7dd5, 0x16a7, 0x7df9,
    0x15e1, 0x7e1c, 0x151b, 0x7e3e, 0x1455, 0x7e5e, 0x138e, 0x7e7e,
    0x12c7, 0x7e9c, 0x1200, 0x7eb9, 0x1139, 0x7ed4, 0x1072, 0x7eef,
    0x0fab, 0x7f08, 0x0ee3, 0x7f20, 0x0e1b, 0x7f37, 0x0d53, 0x7f4c,
    0x0c8b, 0x7f61, 0x0bc3, 0x7f74, 0x0afb, 0x7f86, 0x0a32, 0x7f96,
    0x096a, 0x7fa6, 0x08a1, 0x7fb4, 0x07d9, 0x7fc1, 0x0710, 0x7fcd,
    0x0647, 0x7fd7, 0x057e, 0x7fe0, 0x04b6, 0x7fe8, 0x03ed, 0x7fef,
    0x0324, 0x7ff5, 0x025b, 0x7ff9, 0x0192, 0x7ffc, 0x00c9, 0x7ffe,
    0x0000, 0x7fff, 0xff37, 0x7ffe, 0xfe6e, 0x7ffc, 0xfda5, 0x7ff9,
    0xfcdc, 0x7ff5, 0xfc13, 0x7fef, 0xfb4a, 0x7fe8, 0xfa82, 0x7fe0,
    0xf9b9, 0x7fd7, 0xf8f0, 0x7fcd, 0xf827, 0x7fc1, 0xf75f, 0x7fb4,
    0xf696, 0x7fa6, 0xf5ce, 0x7f96, 0xf505, 0x7f86, 0xf43d, 0x7f74,
    0xf375, 0x7f61, 0xf2ad, 0x7f4c, 0xf1e5, 0x7f37, 0xf11d, 0x7f20,
    0xf055, 0x7f08, 0xef8e, 0x7eef, 0xeec7, 0x7ed4, 0xee00, 0x7eb9,
    0xed39, 0x7e9c, 0xec72, 0x7e7e, 0xebab, 0x7e5e, 0xeae5, 0x7e3e,
    0xea1f, 0x7e1c, 0xe959, 0x7df9, 0xe893, 0x7dd5, 0xe7cd, 0x7db0,
    0xe708, 0x7d89, 0xe643, 0x7d61, 0xe57e, 0x7d38, 0xe4ba, 0x7d0e,
    0xe3f5, 0x7ce2, 0xe331, 0x7cb6, 0xe26d, 0x7c88, 0xe1aa, 0x7c59,
    0xe0e7, 0x7c29, 0xe024, 0x7bf7, 0xdf61, 0x7bc4, 0xde9f, 0x7b91,
    0xdddd, 0x7b5c, 0xdd1c, 0x7b25, 0xdc5a, 0x7aee, 0xdb99, 0x7ab5,
    0xdad9, 0x7a7c, 0xda19, 0x7a41, 0xd959, 0x7a04, 0xd899, 0x79c7,
    0xd7da, 0x7989, 0xd71b, 0x7949, 0xd65d, 0x7908, 0xd59f, 0x78c6,
    0xd4e2, 0x7883, 0xd425, 0x783f, 0xd368, 0x77f9, 0xd2ac, 0x77b3,
    0xd1f0, 0x776b, 0xd134, 0x7722, 0xd07a, 0x76d8, 0xcfbf, 0x768d,
    0xcf05, 0x7640, 0xce4c, 0x75f3, 0xcd93, 0x75a4, 0xccda, 0x7554,
    0xcc22, 0x7503, 0xcb6a, 0x74b1, 0xcab3, 0x745e, 0xc9fd, 0x740a,
    0xc947, 0x73b5, 0xc891, 0x735e, 0xc7dc, 0x7306, 0xc728, 0x72ae,
    0xc674, 0x7254, 0xc5c1, 0x71f9, 0xc50e, 0x719d, 0xc45c, 0x7140,
    0xc3aa, 0x70e1, 0xc2f9, 0x7082, 0xc249, 0x7022, 0xc199, 0x6fc0,
    0xc0ea, 0x6f5e, 0xc03b, 0x6efa, 0xbf8d, 0x6e95, 0xbee0, 0x6e30,
    0xbe33, 0x6dc9, 0xbd87, 0x6d61, 0xbcdb, 0x6cf8, 0xbc30, 0x6c8e,
    0xbb86, 0x6c23, 0xbadd, 0x6bb7, 0xba34, 0x6b4a, 0xb98c, 0x6adb,
    0xb8e4, 0x6a6c, 0xb83d, 0x69fc, 0xb797, 0x698b, 0xb6f2, 0x6919,
    0xb64d, 0x68a5, 0xb5a9, 0x6831, 0xb506, 0x67bc, 0xb463, 0x6745,
    0xb3c1, 0x66ce, 0xb320, 0x6656, 0xb280, 0x65dd, 0xb1e0, 0x6562,
    0xb141, 0x64e7, 0xb0a3, 0x646b, 0xb006, 0x63ee, 0xaf69, 0x6370,
    0xaecd, 0x62f1, 0xae32, 0x6271, 0xad98, 0x61f0, 0xacff, 0x616e,
    0xac66, 0x60eb, 0xabce, 0x6067, 0xab37, 0x5fe2, 0xaaa1, 0x5f5d,
    0xaa0c, 0x5ed6, 0xa977, 0x5e4f, 0xa8e3, 0x5dc6, 0xa850, 0x5d3d,
    0xa7be, 0x5cb3, 0xa72d, 0x5c28, 0xa69d, 0x5b9c, 0xa60d, 0x5b0f,
    0xa57f, 0x5a81, 0xa4f1, 0x59f3, 0xa464, 0x5963, 0xa3d8, 0x58d3,
    0xa34d, 0x5842, 0xa2c3, 0x57b0, 0xa23a, 0x571d, 0xa1b1, 0x5689,
    0xa12a, 0x55f4, 0xa0a3, 0x555f, 0xa01e, 0x54c9, 0x9f99, 0x5432,
    0x9f15, 0x539a, 0x9e92, 0x5301, 0x9e10, 0x5268, 0x9d8f, 0x51ce,
    0x9d0f, 0x5133, 0x9c90, 0x5097, 0x9c12, 0x4ffa, 0x9b95, 0x4f5d,
    0x9b19, 0x4ebf, 0x9a9e, 0x4e20, 0x9a23, 0x4d80, 0x99aa, 0x4ce0,
    0x9932, 0x4c3f, 0x98bb, 0x4b9d, 0x9844, 0x4afa, 0x97cf, 0x4a57,
    0x975b, 0x49b3, 0x96e7, 0x490e, 0x9675, 0x4869, 0x9604, 0x47c3,
    0x9594, 0x471c, 0x9525, 0x4674, 0x94b6, 0x45cc, 0x9449, 0x4523,
    0x93dd, 0x447a, 0x9372, 0x43d0, 0x9308, 0x4325, 0x929f, 0x4279,
    0x9237, 0x41cd, 0x91d0, 0x4120, 0x916b, 0x4073, 0x9106, 0x3fc5,
    0x90a2, 0x3f16, 0x9040, 0x3e67, 0x8fde, 0x3db7, 0x8f7e, 0x3d07,
    0x8f1f, 0x3c56, 0x8ec0, 0x3ba4, 0x8e63, 0x3af2, 0x8e07, 0x3a3f,
    0x8dac, 0x398c, 0x8d52, 0x38d8, 0x8cfa, 0x3824, 0x8ca2, 0x376f,
    0x8c4b, 0x36b9, 0x8bf6, 0x3603, 0x8ba2, 0x354d, 0x8b4f, 0x3496,
    0x8afd, 0x33de, 0x8aac, 0x3326, 0x8a5c, 0x326d, 0x8a0d, 0x31b4,
    0x89c0, 0x30fb, 0x8973, 0x3041, 0x8928, 0x2f86, 0x88de, 0x2ecc,
    0x8895, 0x2e10, 0x884d, 0x2d54, 0x8807, 0x2c98, 0x87c1, 0x2bdb,
    0x877d, 0x2b1e, 0x873a, 0x2a61, 0x86f8, 0x29a3, 0x86b7, 0x28e5,
    0x8677, 0x2826, 0x8639, 0x2767, 0x85fc, 0x26a7, 0x85bf, 0x25e7,
    0x8584, 0x2527, 0x854b, 0x2467, 0x8512, 0x23a6, 0x84db, 0x22e4,
    0x84a4, 0x2223, 0x846f, 0x2161, 0x843c, 0x209f, 0x8409, 0x1fdc,
    0x83d7, 0x1f19, 0x83a7, 0x1e56, 0x8378, 0x1d93, 0x834a, 0x1ccf,
    0x831e, 0x1c0b, 0x82f2, 0x1b46, 0x82c8, 0x1a82, 0x829f, 0x19bd,
    0x8277, 0x18f8, 0x8250, 0x1833, 0x822b, 0x176d, 0x8207, 0x16a7,
    0x81e4, 0x15e1, 0x81c2, 0x151b, 0x81a2, 0x1455, 0x8182, 0x138e,
    0x8164, 0x12c7, 0x8147, 0x1200, 0x812c, 0x1139, 0x8111, 0x1072,
    0x80f8, 0x0fab, 0x80e0, 0x0ee3, 0x80c9, 0x0e1b, 0x80b4, 0x0d53,
    0x809f, 0x0c8b, 0x808c, 0x0bc3, 0x807a, 0x0afb, 0x806a, 0x0a32,
    0x805a, 0x096a, 0x804c, 0x08a1, 0x803f, 0x07d9, 0x8033, 0x0710,
    0x8029, 0x0647, 0x8020, 0x057e, 0x8018, 0x04b6, 0x8011, 0x03ed,
    0x800b, 0x0324, 0x8007, 0x025b, 0x8004, 0x0192, 0x8002, 0x00c9,
    0x8001, 0x0000, 0x8002, 0xff37, 0x8004, 0xfe6e, 0x8007, 0xfda5,
    0x800b, 0xfcdc, 0x8011, 0xfc13, 0x8018, 0xfb4a, 0x8020, 0xfa82,
    0x8029, 0xf9b9, 0x8033, 0xf8f0, 0x803f, 0xf827, 0x804c, 0xf75f,
    0x805a, 0xf696, 0x806a, 0xf5ce, 0x807a, 0xf505, 0x808c, 0xf43d,
    0x809f, 0xf375, 0x80b4, 0xf2ad, 0x80c9, 0xf1e5, 0x80e0, 0xf11d,
    0x80f8, 0xf055, 0x8111, 0xef8e, 0x812c, 0xeec7, 0x8147, 0xee00,
    0x8164, 0xed39, 0x8182, 0xec72, 0x81a2, 0xebab, 0x81c2, 0xeae5,
    0x81e4, 0xea1f, 0x8207, 0xe959, 0x822b, 0xe893, 0x8250, 0xe7cd,
    0x8277, 0xe708, 0x829f, 0xe643, 0x82c8, 0xe57e, 0x82f2, 0xe4ba,
    0x831e, 0xe3f5, 0x834a, 0xe331, 0x8378, 0xe26d, 0x83a7, 0xe1aa,
    0x83d7, 0xe0e7, 0x8409, 0xe024, 0x843c, 0xdf61, 0x846f, 0xde9f,
    0x84a4, 0xdddd, 0x84db, 0xdd1c, 0x8512, 0xdc5a, 0x854b, 0xdb99,
    0x8584, 0xdad9, 0x85bf, 0xda19, 0x85fc, 0xd959, 0x8639, 0xd899,
    0x8677, 0xd7da, 0x86b7, 0xd71b, 0x86f8, 0xd65d, 0x873a, 0xd59f,
    0x877d, 0xd4e2, 0x87c1, 0xd425, 0x8807, 0xd368, 0x884d, 0xd2ac,
    0x8895, 0xd1f0, 0x88de, 0xd134, 0x8928, 0xd07a, 0x8973, 0xcfbf,
    0x89c0, 0xcf05, 0x8a0d, 0xce4c, 0x8a5c, 0xcd93, 0x8aac, 0xccda,
    0x8afd, 0xcc22, 0x8b4f, 0xcb6a, 0x8ba2, 0xcab3, 0x8bf6, 0xc9fd,
    0x8c4b, 0xc947, 0x8ca2, 0xc891, 0x8cfa, 0xc7dc, 0x8d52, 0xc728,
    0x8dac, 0xc674, 0x8e07, 0xc5c1, 0x8e63, 0xc50e, 0x8ec0, 0xc45c,
    0x8f1f, 0xc3aa, 0x8f7e, 0xc2f9, 0x8fde, 0xc249, 0x9040, 0xc199,
    0x90a2, 0xc0ea, 0x9106, 0xc03b, 0x916b, 0xbf8d, 0x91d0, 0xbee0,
    0x9237, 0xbe33, 0x929f, 0xbd87, 0x9308, 0xbcdb, 0x9372, 0xbc30,
    0x93dd, 0xbb86, 0x9449, 0xbadd, 0x94b6, 0xba34, 0x9525, 0xb98c,
    0x9594, 0xb8e4, 0x9604, 0xb83d, 0x9675, 0xb797, 0x96e7, 0xb6f2,
    0x975b, 0xb64d, 0x97cf, 0xb5a9, 0x9844, 0xb506, 0x98bb, 0xb463,
    0x9932, 0xb3c1, 0x99aa, 0xb320, 0x9a23, 0xb280, 0x9a9e, 0xb1e0,
    0x9b19, 0xb141, 0x9b95, 0xb0a3, 0x9c12, 0xb006, 0x9c90, 0xaf69,
    0x9d0f, 0xaecd, 0x9d8f, 0xae32, 0x9e10, 0xad98, 0x9e92, 0xacff,
    0x9f15, 0xac66, 0x9f99, 0xabce, 0xa01e, 0xab37, 0xa0a3, 0xaaa1,
    0xa12a, 0xaa0c, 0xa1b1, 0xa977, 0xa23a, 0xa8e3, 0xa2c3, 0xa850,
    0xa34d, 0xa7be, 0xa3d8, 0xa72d, 0xa464, 0xa69d, 0xa4f1, 0xa60d,
    0xa57f, 0xa57f, 0xa60d, 0xa4f1, 0xa69d, 0xa464, 0xa72d, 0xa3d8,
    0xa7be, 0xa34d, 0xa850, 0xa2c3, 0xa8e3, 0xa23a, 0xa977, 0xa1b1,
    0xaa0c, 0xa12a, 0xaaa1, 0xa0a3, 0xab37, 0xa01e, 0xabce, 0x9f99,
    0xac66, 0x9f15, 0xacff, 0x9e92, 0xad98, 0x9e10, 0xae32, 0x9d8f,
    0xaecd, 0x9d0f, 0xaf69, 0x9c90, 0xb006, 0x9c12, 0xb0a3, 0x9b95,
    0xb141, 0x9b19, 0xb1e0, 0x9a9e, 0xb280, 0x9a23, 0xb320, 0x99aa,
    0xb3c1, 0x9932, 0xb463, 0x98bb, 0xb506, 0x9844, 0xb5a9, 0x97cf,
    0xb64d, 0x975b, 0xb6f2, 0x96e7, 0xb797, 0x9675, 0xb83d, 0x9604,
    0xb8e4, 0x9594, 0xb98c, 0x9525, 0xba34, 0x94b6, 0xbadd, 0x9449,
    0xbb86, 0x93dd, 0xbc30, 0x9372, 0xbcdb, 0x9308, 0xbd87, 0x929f,
    0xbe33, 0x9237, 0xbee0, 0x91d0, 0xbf8d, 0x916b, 0xc03b, 0x9106,
    0xc0ea, 0x90a2, 0xc199, 0x9040, 0xc249, 0x8fde, 0xc2f9, 0x8f7e,
    0xc3aa, 0x8f1f, 0xc45c, 0x8ec0, 0xc50e, 0x8e63, 0xc5c1, 0x8e07,
    0xc674, 0x8dac, 0xc728, 0x8d52, 0xc7dc, 0x8cfa, 0xc891, 0x8ca2,
    0xc947, 0x8c4b, 0xc9fd, 0x8bf6, 0xcab3, 0x8ba2, 0xcb6a, 0x8b4f,
    0xcc22, 0x8afd, 0xccda, 0x8aac, 0xcd93, 0x8a5c, 0xce4c, 0x8a0d,
    0xcf05, 0x89c0, 0xcfbf, 0x8973, 0xd07a, 0x8928, 0xd134, 0x88de,
    0xd1f0, 0x8895, 0xd2ac, 0x884d, 0xd368, 0x8807, 0xd425, 0x87c1,
    0xd4e2, 0x877d, 0xd59f, 0x873a, 0xd65d, 0x86f8, 0xd71b, 0x86b7,
    0xd7da, 0x8677, 0xd899, 0x8639, 0xd959, 0x85fc, 0xda19, 0x85bf,
    0xdad9, 0x8584, 0xdb99, 0x854b, 0xdc5a, 0x8512, 0xdd1c, 0x84db,
    0xdddd, 0x84a4, 0xde9f, 0x846f, 0xdf61, 0x843c, 0xe024, 0x8409,
    0xe0e7, 0x83d7, 0xe1aa, 0x83a7, 0xe26d, 0x8378, 0xe331, 0x834a,
    0xe3f5, 0x831e, 0xe4ba, 0x82f2, 0xe57e, 0x82c8, 0xe643, 0x829f,
    0xe708, 0x8277, 0xe7cd, 0x8250, 0xe893, 0x822b, 0xe959, 0x8207,
    0xea1f, 0x81e4, 0xeae5, 0x81c2, 0xebab, 0x81a2, 0xec72, 0x8182,
    0xed39, 0x8164, 0xee00, 0x8147, 0xeec7, 0x812c, 0xef8e, 0x8111,
    0xf055, 0x80f8, 0xf11d, 0x80e0, 0xf1e5, 0x80c9, 0xf2ad, 0x80b4,
    0xf375, 0x809f, 0xf43d, 0x808c, 0xf505, 0x807a, 0xf5ce, 0x806a,
    0xf696, 0x805a, 0xf75f, 0x804c, 0xf827, 0x803f, 0xf8f0, 0x8033,
    0xf9b9, 0x8029, 0xfa82, 0x8020, 0xfb4a, 0x8018, 0xfc13, 0x8011,
    0xfcdc, 0x800b, 0xfda5, 0x8007, 0xfe6e, 0x8004, 0xff37, 0x8002,
    };
// Pseudo code for generation of the Twiddle A and B coefficients...
// for (i=0; i<FFT_N; i++)
// {
//     twiddleCoef_A[2*i]   = 0.5 * (1.0 - sin (2 * PI / (double)(2 * FFT_N) * (double)i));
//     twiddleCoef_A[2*i+1] = 0.5 * (-1.0 * cos (2 * PI / (double)(2 * FFT_N) * (double)i));
//     twiddleCoef_B[2*i]   = 0.5 * (1.0 + sin (2 * PI / (double)(2 * FFT_N) * (double)i));
//     twiddleCoef_B[2*i+1] = 0.5 * (1.0 * cos (2 * PI / (double)(2 * FFT_N) * (double)i));
// }
// Twiddle A and B coefficients for 16-bit 1024-bin FFT (used for 512-bin result)
static const uint16_t twiddlesA[2048] = {
    0x3fff, 0xc001, 0x3fcd, 0xc001, 0x3f9a, 0xc001, 0x3f68, 0xc002,
    0x3f36, 0xc002, 0x3f04, 0xc003, 0x3ed1, 0xc004, 0x3e9f, 0xc005,
    0x3e6d, 0xc006, 0x3e3b, 0xc007, 0x3e08, 0xc009, 0x3dd6, 0xc00a,
    0x3da4, 0xc00c, 0x3d72, 0xc00e, 0x3d40, 0xc010, 0x3d0d, 0xc012,
    0x3cdb, 0xc015, 0x3ca9, 0xc017, 0x3c77, 0xc01a, 0x3c45, 0xc01d,
    0x3c12, 0xc020, 0x3be0, 0xc023, 0x3bae, 0xc026, 0x3b7c, 0xc02a,
    0x3b4a, 0xc02d, 0x3b18, 0xc031, 0x3ae6, 0xc035, 0x3ab3, 0xc039,
    0x3a81, 0xc03d, 0x3a4f, 0xc042, 0x3a1d, 0xc046, 0x39eb, 0xc04b,
    0x39b9, 0xc050, 0x3987, 0xc055, 0x3955, 0xc05a, 0x3923, 0xc05f,
    0x38f1, 0xc065, 0x38bf, 0xc06a, 0x388d, 0xc070, 0x385b, 0xc076,
    0x3829, 0xc07c, 0x37f8, 0xc082, 0x37c6, 0xc089, 0x3794, 0xc08f,
    0x3762, 0xc096, 0x3730, 0xc09d, 0x36ff, 0xc0a4, 0x36cd, 0xc0ab,
    0x369b, 0xc0b2, 0x3669, 0xc0ba, 0x3638, 0xc0c1, 0x3606, 0xc0c9,
    0x35d4, 0xc0d1, 0x35a3, 0xc0d9, 0x3571, 0xc0e1, 0x3540, 0xc0ea,
    0x350e, 0xc0f2, 0x34dd, 0xc0fb, 0x34ab, 0xc104, 0x347a, 0xc10d,
    0x3448, 0xc116, 0x3417, 0xc11f, 0x33e5, 0xc128, 0x33b4, 0xc132,
    0x3383, 0xc13c, 0x3351, 0xc146, 0x3320, 0xc150, 0x32ef, 0xc15a,
    0x32be, 0xc164, 0x328d, 0xc16f, 0x325c, 0xc179, 0x322a, 0xc184,
    0x31f9, 0xc18f, 0x31c8, 0xc19a, 0x3197, 0xc1a5, 0x3166, 0xc1b1,
    0x3135, 0xc1bc, 0x3105, 0xc1c8, 0x30d4, 0xc1d4, 0x30a3, 0xc1e0,
    0x3072, 0xc1ec, 0x3041, 0xc1f8, 0x3011, 0xc205, 0x2fe0, 0xc211,
    0x2faf, 0xc21e, 0x2f7f, 0xc22b, 0x2f4e, 0xc238, 0x2f1e, 0xc245,
    0x2eed, 0xc252, 0x2ebd, 0xc260, 0x2e8d, 0xc26e, 0x2e5c, 0xc27b,
    0x2e2c, 0xc289, 0x2dfc, 0xc297, 0x2dcb, 0xc2a6, 0x2d9b, 0xc2b4,
    0x2d6b, 0xc2c2, 0x2d3b, 0xc2d1, 0x2d0b, 0xc2e0, 0x2cdb, 0xc2ef,
    0x2cab, 0xc2fe, 0x2c7b, 0xc30d, 0x2c4b, 0xc31d, 0x2c1c, 0xc32c,
    0x2bec, 0xc33c, 0x2bbc, 0xc34c, 0x2b8c, 0xc35c, 0x2b5d, 0xc36c,
    0x2b2d, 0xc37c, 0x2afe, 0xc38d, 0x2ace, 0xc39d, 0x2a9f, 0xc3ae,
    0x2a70, 0xc3bf, 0x2a40, 0xc3d0, 0x2a11, 0xc3e1, 0x29e2, 0xc3f2,
    0x29b3, 0xc404, 0x2984, 0xc415, 0x2955, 0xc427, 0x2926, 0xc439,
    0x28f7, 0xc44b, 0x28c8, 0xc45d, 0x2899, 0xc46f, 0x286a, 0xc482,
    0x283c, 0xc494, 0x280d, 0xc4a7, 0x27de, 0xc4ba, 0x27b0, 0xc4cd,
    0x2781, 0xc4e0, 0x2753, 0xc4f3, 0x2725, 0xc507, 0x26f6, 0xc51a,
    0x26c8, 0xc52e, 0x269a, 0xc542, 0x266c, 0xc556, 0x263e, 0xc56a,
    0x2610, 0xc57f, 0x25e2, 0xc593, 0x25b4, 0xc5a8, 0x2586, 0xc5bc,
    0x2558, 0xc5d1, 0x252b, 0xc5e6, 0x24fd, 0xc5fb, 0x24d0, 0xc611,
    0x24a2, 0xc626, 0x2475, 0xc63c, 0x2447, 0xc651, 0x241a, 0xc667,
    0x23ed, 0xc67d, 0x23c0, 0xc693, 0x2393, 0xc6a9, 0x2366, 0xc6c0,
    0x2339, 0xc6d6, 0x230c, 0xc6ed, 0x22df, 0xc704, 0x22b2, 0xc71b,
    0x2286, 0xc732, 0x2259, 0xc749, 0x222d, 0xc760, 0x2200, 0xc778,
    0x21d4, 0xc790, 0x21a8, 0xc7a7, 0x217b, 0xc7bf, 0x214f, 0xc7d7,
    0x2123, 0xc7ef, 0x20f7, 0xc808, 0x20cb, 0xc820, 0x209f, 0xc839,
    0x2074, 0xc851, 0x2048, 0xc86a, 0x201c, 0xc883, 0x1ff1, 0xc89c,
    0x1fc5, 0xc8b6, 0x1f9a, 0xc8cf, 0x1f6f, 0xc8e8, 0x1f43, 0xc902,
    0x1f18, 0xc91c, 0x1eed, 0xc936, 0x1ec2, 0xc950, 0x1e97, 0xc96a,
    0x1e6c, 0xc984, 0x1e42, 0xc99f, 0x1e17, 0xc9b9, 0x1dec, 0xc9d4,
    0x1dc2, 0xc9ef, 0x1d97, 0xca0a, 0x1d6d, 0xca25, 0x1d43, 0xca40,
    0x1d19, 0xca5b, 0x1cef, 0xca77, 0x1cc5, 0xca93, 0x1c9b, 0xcaae,
    0x1c71, 0xcaca, 0x1c47, 0xcae6, 0x1c1d, 0xcb02, 0x1bf4, 0xcb1e,
    0x1bca, 0xcb3b, 0x1ba1, 0xcb57, 0x1b78, 0xcb74, 0x1b4e, 0xcb91,
    0x1b25, 0xcbae, 0x1afc, 0xcbcb, 0x1ad3, 0xcbe8, 0x1aaa, 0xcc05,
    0x1a82, 0xcc22, 0x1a59, 0xcc40, 0x1a30, 0xcc5e, 0x1a08, 0xcc7b,
    0x19df, 0xcc99, 0x19b7, 0xccb7, 0x198f, 0xccd5, 0x1967, 0xccf4,
    0x193f, 0xcd12, 0x1917, 0xcd30, 0x18ef, 0xcd4f, 0x18c7, 0xcd6e,
    0x189f, 0xcd8d, 0x1878, 0xcdac, 0x1850, 0xcdcb, 0x1829, 0xcdea,
    0x1802, 0xce09, 0x17da, 0xce29, 0x17b3, 0xce48, 0x178c, 0xce68,
    0x1765, 0xce88, 0x173f, 0xcea8, 0x1718, 0xcec8, 0x16f1, 0xcee8,
    0x16cb, 0xcf08, 0x16a4, 0xcf29, 0x167e, 0xcf49, 0x1658, 0xcf6a,
    0x1632, 0xcf8b, 0x160c, 0xcfac, 0x15e6, 0xcfcd, 0x15c0, 0xcfee,
    0x159a, 0xd00f, 0x1575, 0xd030, 0x154f, 0xd052, 0x152a, 0xd073,
    0x1505, 0xd095, 0x14df, 0xd0b7, 0x14ba, 0xd0d9, 0x1495, 0xd0fb,
    0x1470, 0xd11d, 0x144c, 0xd13f, 0x1427, 0xd162, 0x1402, 0xd184,
    0x13de, 0xd1a7, 0x13ba, 0xd1ca, 0x1395, 0xd1ec, 0x1371, 0xd20f,
    0x134d, 0xd232, 0x1329, 0xd255, 0x1305, 0xd279, 0x12e2, 0xd29c,
    0x12be, 0xd2c0, 0x129b, 0xd2e3, 0x1277, 0xd307, 0x1254, 0xd32b,
    0x1231, 0xd34f, 0x120e, 0xd373, 0x11eb, 0xd397, 0x11c8, 0xd3bb,
    0x11a5, 0xd3df, 0x1183, 0xd404, 0x1160, 0xd428, 0x113e, 0xd44d,
    0x111c, 0xd472, 0x10f9, 0xd497, 0x10d7, 0xd4bc, 0x10b5, 0xd4e1,
    0x1094, 0xd506, 0x1072, 0xd52b, 0x1050, 0xd551, 0x102f, 0xd576,
    0x100e, 0xd59c, 0x0fec, 0xd5c2, 0x0fcb, 0xd5e7, 0x0faa, 0xd60d,
    0x0f89, 0xd633, 0x0f68, 0xd659, 0x0f48, 0xd680, 0x0f27, 0xd6a6,
    0x0f07, 0xd6cc, 0x0ee7, 0xd6f3, 0x0ec6, 0xd719, 0x0ea6, 0xd740,
    0x0e86, 0xd767, 0x0e67, 0xd78e, 0x0e47, 0xd7b5, 0x0e27, 0xd7dc,
    0x0e08, 0xd803, 0x0de8, 0xd82a, 0x0dc9, 0xd852, 0x0daa, 0xd879,
    0x0d8b, 0xd8a1, 0x0d6c, 0xd8c9, 0x0d4e, 0xd8f0, 0x0d2f, 0xd918,
    0x0d10, 0xd940, 0x0cf2, 0xd968, 0x0cd4, 0xd990, 0x0cb6, 0xd9b9,
    0x0c98, 0xd9e1, 0x0c7a, 0xda09, 0x0c5c, 0xda32, 0x0c3e, 0xda5a,
    0x0c21, 0xda83, 0x0c03, 0xdaac, 0x0be6, 0xdad5, 0x0bc9, 0xdafe,
    0x0bac, 0xdb27, 0x0b8f, 0xdb50, 0x0b72, 0xdb79, 0x0b56, 0xdba2,
    0x0b39, 0xdbcc, 0x0b1d, 0xdbf5, 0x0b01, 0xdc1f, 0x0ae5, 0xdc49,
    0x0ac9, 0xdc72, 0x0aad, 0xdc9c, 0x0a91, 0xdcc6, 0x0a75, 0xdcf0,
    0x0a5a, 0xdd1a, 0x0a3f, 0xdd44, 0x0a23, 0xdd6f, 0x0a08, 0xdd99,
    0x09ed, 0xddc3, 0x09d3, 0xddee, 0x09b8, 0xde18, 0x099d, 0xde43,
    0x0983, 0xde6e, 0x0969, 0xde99, 0x094e, 0xdec4, 0x0934, 0xdeef,
    0x091a, 0xdf1a, 0x0901, 0xdf45, 0x08e7, 0xdf70, 0x08cd, 0xdf9b,
    0x08b4, 0xdfc7, 0x089b, 0xdff2, 0x0882, 0xe01e, 0x0869, 0xe049,
    0x0850, 0xe075, 0x0837, 0xe0a1, 0x081f, 0xe0cd, 0x0806, 0xe0f9,
    0x07ee, 0xe125, 0x07d6, 0xe151, 0x07be, 0xe17d, 0x07a6, 0xe1a9,
    0x078e, 0xe1d5, 0x0776, 0xe202, 0x075f, 0xe22e, 0x0748, 0xe25b,
    0x0730, 0xe287, 0x0719, 0xe2b4, 0x0702, 0xe2e1, 0x06ec, 0xe30d,
    0x06d5, 0xe33a, 0x06be, 0xe367, 0x06a8, 0xe394, 0x0692, 0xe3c1,
    0x067c, 0xe3ee, 0x0666, 0xe41c, 0x0650, 0xe449, 0x063a, 0xe476,
    0x0624, 0xe4a4, 0x060f, 0xe4d1, 0x05fa, 0xe4ff, 0x05e5, 0xe52c,
    0x05d0, 0xe55a, 0x05bb, 0xe588, 0x05a6, 0xe5b5, 0x0591, 0xe5e3,
    0x057d, 0xe611, 0x0569, 0xe63f, 0x0555, 0xe66d, 0x0541, 0xe69b,
    0x052d, 0xe6ca, 0x0519, 0xe6f8, 0x0505, 0xe726, 0x04f2, 0xe754,
    0x04df, 0xe783, 0x04cb, 0xe7b1, 0x04b8, 0xe7e0, 0x04a6, 0xe80e,
    0x0493, 0xe83d, 0x0480, 0xe86c, 0x046e, 0xe89a, 0x045b, 0xe8c9,
    0x0449, 0xe8f8, 0x0437, 0xe927, 0x0425, 0xe956, 0x0414, 0xe985,
    0x0402, 0xe9b4, 0x03f1, 0xe9e3, 0x03df, 0xea13, 0x03ce, 0xea42,
    0x03bd, 0xea71, 0x03ac, 0xeaa0, 0x039c, 0xead0, 0x038b, 0xeaff,
    0x037b, 0xeb2f, 0x036a, 0xeb5e, 0x035a, 0xeb8e, 0x034a, 0xebbe,
    0x033a, 0xebed, 0x032b, 0xec1d, 0x031b, 0xec4d, 0x030c, 0xec7d,
    0x02fd, 0xecad, 0x02ed, 0xecdd, 0x02de, 0xed0d, 0x02d0, 0xed3d,
    0x02c1, 0xed6d, 0x02b2, 0xed9d, 0x02a4, 0xedcd, 0x0296, 0xedfd,
    0x0288, 0xee2d, 0x027a, 0xee5e, 0x026c, 0xee8e, 0x025e, 0xeebe,
    0x0251, 0xeeef, 0x0244, 0xef1f, 0x0236, 0xef50, 0x0229, 0xef80,
    0x021d, 0xefb1, 0x0210, 0xefe2, 0x0203, 0xf012, 0x01f7, 0xf043,
    0x01ea, 0xf074, 0x01de, 0xf0a4, 0x01d2, 0xf0d5, 0x01c7, 0xf106,
    0x01bb, 0xf137, 0x01af, 0xf168, 0x01a4, 0xf199, 0x0199, 0xf1ca,
    0x018e, 0xf1fb, 0x0183, 0xf22c, 0x0178, 0xf25d, 0x016d, 0xf28e,
    0x0163, 0xf2bf, 0x0158, 0xf2f0, 0x014e, 0xf322, 0x0144, 0xf353,
    0x013a, 0xf384, 0x0131, 0xf3b6, 0x0127, 0xf3e7, 0x011e, 0xf418,
    0x0114, 0xf44a, 0x010b, 0xf47b, 0x0102, 0xf4ad, 0x00f9, 0xf4de,
    0x00f1, 0xf510, 0x00e8, 0xf541, 0x00e0, 0xf573, 0x00d8, 0xf5a4,
    0x00d0, 0xf5d6, 0x00c8, 0xf607, 0x00c0, 0xf639, 0x00b8, 0xf66b,
    0x00b1, 0xf69d, 0x00aa, 0xf6ce, 0x00a2, 0xf700, 0x009b, 0xf732,
    0x0095, 0xf764, 0x008e, 0xf795, 0x0087, 0xf7c7, 0x0081, 0xf7f9,
    0x007b, 0xf82b, 0x0075, 0xf85d, 0x006f, 0xf88f, 0x0069, 0xf8c1,
    0x0063, 0xf8f3, 0x005e, 0xf925, 0x0059, 0xf957, 0x0053, 0xf989,
    0x004e, 0xf9bb, 0x004a, 0xf9ed, 0x0045, 0xfa1f, 0x0040, 0xfa51,
    0x003c, 0xfa83, 0x0038, 0xfab5, 0x0034, 0xfae7, 0x0030, 0xfb19,
    0x002c, 0xfb4b, 0x0028, 0xfb7d, 0x0025, 0xfbb0, 0x0021, 0xfbe2,
    0x001e, 0xfc14, 0x001b, 0xfc46, 0x0018, 0xfc78, 0x0016, 0xfcaa,
    0x0013, 0xfcdd, 0x0011, 0xfd0f, 0x000f, 0xfd41, 0x000d, 0xfd73,
    0x000b, 0xfda5, 0x0009, 0xfdd8, 0x0007, 0xfe0a, 0x0006, 0xfe3c,
    0x0004, 0xfe6e, 0x0003, 0xfea1, 0x0002, 0xfed3, 0x0001, 0xff05,
    0x0001, 0xff37, 0x0000, 0xff6a, 0x0000, 0xff9c, 0x0000, 0xffce,
    0x0000, 0x0000, 0x0000, 0x0032, 0x0000, 0x0064, 0x0000, 0x0096,
    0x0001, 0x00c9, 0x0001, 0x00fb, 0x0002, 0x012d, 0x0003, 0x015f,
    0x0004, 0x0192, 0x0006, 0x01c4, 0x0007, 0x01f6, 0x0009, 0x0228,
    0x000b, 0x025b, 0x000d, 0x028d, 0x000f, 0x02bf, 0x0011, 0x02f1,
    0x0013, 0x0323, 0x0016, 0x0356, 0x0018, 0x0388, 0x001b, 0x03ba,
    0x001e, 0x03ec, 0x0021, 0x041e, 0x0025, 0x0450, 0x0028, 0x0483,
    0x002c, 0x04b5, 0x0030, 0x04e7, 0x0034, 0x0519, 0x0038, 0x054b,
    0x003c, 0x057d, 0x0040, 0x05af, 0x0045, 0x05e1, 0x004a, 0x0613,
    0x004e, 0x0645, 0x0053, 0x0677, 0x0059, 0x06a9, 0x005e, 0x06db,
    0x0063, 0x070d, 0x0069, 0x073f, 0x006f, 0x0771, 0x0075, 0x07a3,
    0x007b, 0x07d5, 0x0081, 0x0807, 0x0087, 0x0839, 0x008e, 0x086b,
    0x0095, 0x089c, 0x009b, 0x08ce, 0x00a2, 0x0900, 0x00aa, 0x0932,
    0x00b1, 0x0963, 0x00b8, 0x0995, 0x00c0, 0x09c7, 0x00c8, 0x09f9,
    0x00d0, 0x0a2a, 0x00d8, 0x0a5c, 0x00e0, 0x0a8d, 0x00e8, 0x0abf,
    0x00f1, 0x0af0, 0x00f9, 0x0b22, 0x0102, 0x0b53, 0x010b, 0x0b85,
    0x0114, 0x0bb6, 0x011e, 0x0be8, 0x0127, 0x0c19, 0x0131, 0x0c4a,
    0x013a, 0x0c7c, 0x0144, 0x0cad, 0x014e, 0x0cde, 0x0158, 0x0d10,
    0x0163, 0x0d41, 0x016d, 0x0d72, 0x0178, 0x0da3, 0x0183, 0x0dd4,
    0x018e, 0x0e05, 0x0199, 0x0e36, 0x01a4, 0x0e67, 0x01af, 0x0e98,
    0x01bb, 0x0ec9, 0x01c7, 0x0efa, 0x01d2, 0x0f2b, 0x01de, 0x0f5c,
    0x01ea, 0x0f8c, 0x01f7, 0x0fbd, 0x0203, 0x0fee, 0x0210, 0x101e,
    0x021d, 0x104f, 0x0229, 0x1080, 0x0236, 0x10b0, 0x0244, 0x10e1,
    0x0251, 0x1111, 0x025e, 0x1142, 0x026c, 0x1172, 0x027a, 0x11a2,
    0x0288, 0x11d3, 0x0296, 0x1203, 0x02a4, 0x1233, 0x02b2, 0x1263,
    0x02c1, 0x1293, 0x02d0, 0x12c3, 0x02de, 0x12f3, 0x02ed, 0x1323,
    0x02fd, 0x1353, 0x030c, 0x1383, 0x031b, 0x13b3, 0x032b, 0x13e3,
    0x033a, 0x1413, 0x034a, 0x1442, 0x035a, 0x1472, 0x036a, 0x14a2,
    0x037b, 0x14d1, 0x038b, 0x1501, 0x039c, 0x1530, 0x03ac, 0x1560,
    0x03bd, 0x158f, 0x03ce, 0x15be, 0x03df, 0x15ed, 0x03f1, 0x161d,
    0x0402, 0x164c, 0x0414, 0x167b, 0x0425, 0x16aa, 0x0437, 0x16d9,
    0x0449, 0x1708, 0x045b, 0x1737, 0x046e, 0x1766, 0x0480, 0x1794,
    0x0493, 0x17c3, 0x04a6, 0x17f2, 0x04b8, 0x1820, 0x04cb, 0x184f,
    0x04df, 0x187d, 0x04f2, 0x18ac, 0x0505, 0x18da, 0x0519, 0x1908,
    0x052d, 0x1936, 0x0541, 0x1965, 0x0555, 0x1993, 0x0569, 0x19c1,
    0x057d, 0x19ef, 0x0591, 0x1a1d, 0x05a6, 0x1a4b, 0x05bb, 0x1a78,
    0x05d0, 0x1aa6, 0x05e5, 0x1ad4, 0x05fa, 0x1b01, 0x060f, 0x1b2f,
    0x0624, 0x1b5c, 0x063a, 0x1b8a, 0x0650, 0x1bb7, 0x0666, 0x1be4,
    0x067c, 0x1c12, 0x0692, 0x1c3f, 0x06a8, 0x1c6c, 0x06be, 0x1c99,
    0x06d5, 0x1cc6, 0x06ec, 0x1cf3, 0x0702, 0x1d1f, 0x0719, 0x1d4c,
    0x0730, 0x1d79, 0x0748, 0x1da5, 0x075f, 0x1dd2, 0x0776, 0x1dfe,
    0x078e, 0x1e2b, 0x07a6, 0x1e57, 0x07be, 0x1e83, 0x07d6, 0x1eaf,
    0x07ee, 0x1edb, 0x0806, 0x1f07, 0x081f, 0x1f33, 0x0837, 0x1f5f,
    0x0850, 0x1f8b, 0x0869, 0x1fb7, 0x0882, 0x1fe2, 0x089b, 0x200e,
    0x08b4, 0x2039, 0x08cd, 0x2065, 0x08e7, 0x2090, 0x0901, 0x20bb,
    0x091a, 0x20e6, 0x0934, 0x2111, 0x094e, 0x213c, 0x0969, 0x2167,
    0x0983, 0x2192, 0x099d, 0x21bd, 0x09b8, 0x21e8, 0x09d3, 0x2212,
    0x09ed, 0x223d, 0x0a08, 0x2267, 0x0a23, 0x2291, 0x0a3f, 0x22bc,
    0x0a5a, 0x22e6, 0x0a75, 0x2310, 0x0a91, 0x233a, 0x0aad, 0x2364,
    0x0ac9, 0x238e, 0x0ae5, 0x23b7, 0x0b01, 0x23e1, 0x0b1d, 0x240b,
    0x0b39, 0x2434, 0x0b56, 0x245e, 0x0b72, 0x2487, 0x0b8f, 0x24b0,
    0x0bac, 0x24d9, 0x0bc9, 0x2502, 0x0be6, 0x252b, 0x0c03, 0x2554,
    0x0c21, 0x257d, 0x0c3e, 0x25a6, 0x0c5c, 0x25ce, 0x0c7a, 0x25f7,
    0x0c98, 0x261f, 0x0cb6, 0x2647, 0x0cd4, 0x2670, 0x0cf2, 0x2698,
    0x0d10, 0x26c0, 0x0d2f, 0x26e8, 0x0d4e, 0x2710, 0x0d6c, 0x2737,
    0x0d8b, 0x275f, 0x0daa, 0x2787, 0x0dc9, 0x27ae, 0x0de8, 0x27d6,
    0x0e08, 0x27fd, 0x0e27, 0x2824, 0x0e47, 0x284b, 0x0e67, 0x2872,
    0x0e86, 0x2899, 0x0ea6, 0x28c0, 0x0ec6, 0x28e7, 0x0ee7, 0x290d,
    0x0f07, 0x2934, 0x0f27, 0x295a, 0x0f48, 0x2980, 0x0f68, 0x29a7,
    0x0f89, 0x29cd, 0x0faa, 0x29f3, 0x0fcb, 0x2a19, 0x0fec, 0x2a3e,
    0x100e, 0x2a64, 0x102f, 0x2a8a, 0x1050, 0x2aaf, 0x1072, 0x2ad5,
    0x1094, 0x2afa, 0x10b5, 0x2b1f, 0x10d7, 0x2b44, 0x10f9, 0x2b69,
    0x111c, 0x2b8e, 0x113e, 0x2bb3, 0x1160, 0x2bd8, 0x1183, 0x2bfc,
    0x11a5, 0x2c21, 0x11c8, 0x2c45, 0x11eb, 0x2c69, 0x120e, 0x2c8d,
    0x1231, 0x2cb1, 0x1254, 0x2cd5, 0x1277, 0x2cf9, 0x129b, 0x2d1d,
    0x12be, 0x2d40, 0x12e2, 0x2d64, 0x1305, 0x2d87, 0x1329, 0x2dab,
    0x134d, 0x2dce, 0x1371, 0x2df1, 0x1395, 0x2e14, 0x13ba, 0x2e36,
    0x13de, 0x2e59, 0x1402, 0x2e7c, 0x1427, 0x2e9e, 0x144c, 0x2ec1,
    0x1470, 0x2ee3, 0x1495, 0x2f05, 0x14ba, 0x2f27, 0x14df, 0x2f49,
    0x1505, 0x2f6b, 0x152a, 0x2f8d, 0x154f, 0x2fae, 0x1575, 0x2fd0,
    0x159a, 0x2ff1, 0x15c0, 0x3012, 0x15e6, 0x3033, 0x160c, 0x3054,
    0x1632, 0x3075, 0x1658, 0x3096, 0x167e, 0x30b7, 0x16a4, 0x30d7,
    0x16cb, 0x30f8, 0x16f1, 0x3118, 0x1718, 0x3138, 0x173f, 0x3158,
    0x1765, 0x3178, 0x178c, 0x3198, 0x17b3, 0x31b8, 0x17da, 0x31d7,
    0x1802, 0x31f7, 0x1829, 0x3216, 0x1850, 0x3235, 0x1878, 0x3254,
    0x189f, 0x3273, 0x18c7, 0x3292, 0x18ef, 0x32b1, 0x1917, 0x32d0,
    0x193f, 0x32ee, 0x1967, 0x330c, 0x198f, 0x332b, 0x19b7, 0x3349,
    0x19df, 0x3367, 0x1a08, 0x3385, 0x1a30, 0x33a2, 0x1a59, 0x33c0,
    0x1a82, 0x33de, 0x1aaa, 0x33fb, 0x1ad3, 0x3418, 0x1afc, 0x3435,
    0x1b25, 0x3452, 0x1b4e, 0x346f, 0x1b78, 0x348c, 0x1ba1, 0x34a9,
    0x1bca, 0x34c5, 0x1bf4, 0x34e2, 0x1c1d, 0x34fe, 0x1c47, 0x351a,
    0x1c71, 0x3536, 0x1c9b, 0x3552, 0x1cc5, 0x356d, 0x1cef, 0x3589,
    0x1d19, 0x35a5, 0x1d43, 0x35c0, 0x1d6d, 0x35db, 0x1d97, 0x35f6,
    0x1dc2, 0x3611, 0x1dec, 0x362c, 0x1e17, 0x3647, 0x1e42, 0x3661,
    0x1e6c, 0x367c, 0x1e97, 0x3696, 0x1ec2, 0x36b0, 0x1eed, 0x36ca,
    0x1f18, 0x36e4, 0x1f43, 0x36fe, 0x1f6f, 0x3718, 0x1f9a, 0x3731,
    0x1fc5, 0x374a, 0x1ff1, 0x3764, 0x201c, 0x377d, 0x2048, 0x3796,
    0x2074, 0x37af, 0x209f, 0x37c7, 0x20cb, 0x37e0, 0x20f7, 0x37f8,
    0x2123, 0x3811, 0x214f, 0x3829, 0x217b, 0x3841, 0x21a8, 0x3859,
    0x21d4, 0x3870, 0x2200, 0x3888, 0x222d, 0x38a0, 0x2259, 0x38b7,
    0x2286, 0x38ce, 0x22b2, 0x38e5, 0x22df, 0x38fc, 0x230c, 0x3913,
    0x2339, 0x392a, 0x2366, 0x3940, 0x2393, 0x3957, 0x23c0, 0x396d,
    0x23ed, 0x3983, 0x241a, 0x3999, 0x2447, 0x39af, 0x2475, 0x39c4,
    0x24a2, 0x39da, 0x24d0, 0x39ef, 0x24fd, 0x3a05, 0x252b, 0x3a1a,
    0x2558, 0x3a2f, 0x2586, 0x3a44, 0x25b4, 0x3a58, 0x25e2, 0x3a6d,
    0x2610, 0x3a81, 0x263e, 0x3a96, 0x266c, 0x3aaa, 0x269a, 0x3abe,
    0x26c8, 0x3ad2, 0x26f6, 0x3ae6, 0x2725, 0x3af9, 0x2753, 0x3b0d,
    0x2781, 0x3b20, 0x27b0, 0x3b33, 0x27de, 0x3b46, 0x280d, 0x3b59,
    0x283c, 0x3b6c, 0x286a, 0x3b7e, 0x2899, 0x3b91, 0x28c8, 0x3ba3,
    0x28f7, 0x3bb5, 0x2926, 0x3bc7, 0x2955, 0x3bd9, 0x2984, 0x3beb,
    0x29b3, 0x3bfc, 0x29e2, 0x3c0e, 0x2a11, 0x3c1f, 0x2a40, 0x3c30,
    0x2a70, 0x3c41, 0x2a9f, 0x3c52, 0x2ace, 0x3c63, 0x2afe, 0x3c73,
    0x2b2d, 0x3c84, 0x2b5d, 0x3c94, 0x2b8c, 0x3ca4, 0x2bbc, 0x3cb4,
    0x2bec, 0x3cc4, 0x2c1c, 0x3cd4, 0x2c4b, 0x3ce3, 0x2c7b, 0x3cf3,
    0x2cab, 0x3d02, 0x2cdb, 0x3d11, 0x2d0b, 0x3d20, 0x2d3b, 0x3d2f,
    0x2d6b, 0x3d3e, 0x2d9b, 0x3d4c, 0x2dcb, 0x3d5a, 0x2dfc, 0x3d69,
    0x2e2c, 0x3d77, 0x2e5c, 0x3d85, 0x2e8d, 0x3d92, 0x2ebd, 0x3da0,
    0x2eed, 0x3dae, 0x2f1e, 0x3dbb, 0x2f4e, 0x3dc8, 0x2f7f, 0x3dd5,
    0x2faf, 0x3de2, 0x2fe0, 0x3def, 0x3011, 0x3dfb, 0x3041, 0x3e08,
    0x3072, 0x3e14, 0x30a3, 0x3e20, 0x30d4, 0x3e2c, 0x3105, 0x3e38,
    0x3135, 0x3e44, 0x3166, 0x3e4f, 0x3197, 0x3e5b, 0x31c8, 0x3e66,
    0x31f9, 0x3e71, 0x322a, 0x3e7c, 0x325c, 0x3e87, 0x328d, 0x3e91,
    0x32be, 0x3e9c, 0x32ef, 0x3ea6, 0x3320, 0x3eb0, 0x3351, 0x3eba,
    0x3383, 0x3ec4, 0x33b4, 0x3ece, 0x33e5, 0x3ed8, 0x3417, 0x3ee1,
    0x3448, 0x3eea, 0x347a, 0x3ef3, 0x34ab, 0x3efc, 0x34dd, 0x3f05,
    0x350e, 0x3f0e, 0x3540, 0x3f16, 0x3571, 0x3f1f, 0x35a3, 0x3f27,
    0x35d4, 0x3f2f, 0x3606, 0x3f37, 0x3638, 0x3f3f, 0x3669, 0x3f46,
    0x369b, 0x3f4e, 0x36cd, 0x3f55, 0x36ff, 0x3f5c, 0x3730, 0x3f63,
    0x3762, 0x3f6a, 0x3794, 0x3f71, 0x37c6, 0x3f77, 0x37f8, 0x3f7e,
    0x3829, 0x3f84, 0x385b, 0x3f8a, 0x388d, 0x3f90, 0x38bf, 0x3f96,
    0x38f1, 0x3f9b, 0x3923, 0x3fa1, 0x3955, 0x3fa6, 0x3987, 0x3fab,
    0x39b9, 0x3fb0, 0x39eb, 0x3fb5, 0x3a1d, 0x3fba, 0x3a4f, 0x3fbe,
    0x3a81, 0x3fc3, 0x3ab3, 0x3fc7, 0x3ae6, 0x3fcb, 0x3b18, 0x3fcf,
    0x3b4a, 0x3fd3, 0x3b7c, 0x3fd6, 0x3bae, 0x3fda, 0x3be0, 0x3fdd,
    0x3c12, 0x3fe0, 0x3c45, 0x3fe3, 0x3c77, 0x3fe6, 0x3ca9, 0x3fe9,
    0x3cdb, 0x3feb, 0x3d0d, 0x3fee, 0x3d40, 0x3ff0, 0x3d72, 0x3ff2,
    0x3da4, 0x3ff4, 0x3dd6, 0x3ff6, 0x3e08, 0x3ff7, 0x3e3b, 0x3ff9,
    0x3e6d, 0x3ffa, 0x3e9f, 0x3ffb, 0x3ed1, 0x3ffc, 0x3f04, 0x3ffd,
    0x3f36, 0x3ffe, 0x3f68, 0x3ffe, 0x3f9a, 0x3fff, 0x3fcd, 0x3fff,
    };

//Twiddle B
static const uint16_t twiddlesB[2048] = {
    0x3fff, 0x3fff, 0x4031, 0x3fff, 0x4064, 0x3fff, 0x4096, 0x3ffe,
    0x40c8, 0x3ffe, 0x40fa, 0x3ffd, 0x412d, 0x3ffc, 0x415f, 0x3ffb,
    0x4191, 0x3ffa, 0x41c3, 0x3ff9, 0x41f6, 0x3ff7, 0x4228, 0x3ff6,
    0x425a, 0x3ff4, 0x428c, 0x3ff2, 0x42be, 0x3ff0, 0x42f1, 0x3fee,
    0x4323, 0x3feb, 0x4355, 0x3fe9, 0x4387, 0x3fe6, 0x43b9, 0x3fe3,
    0x43ec, 0x3fe0, 0x441e, 0x3fdd, 0x4450, 0x3fda, 0x4482, 0x3fd6,
    0x44b4, 0x3fd3, 0x44e6, 0x3fcf, 0x4518, 0x3fcb, 0x454b, 0x3fc7,
    0x457d, 0x3fc3, 0x45af, 0x3fbe, 0x45e1, 0x3fba, 0x4613, 0x3fb5,
    0x4645, 0x3fb0, 0x4677, 0x3fab, 0x46a9, 0x3fa6, 0x46db, 0x3fa1,
    0x470d, 0x3f9b, 0x473f, 0x3f96, 0x4771, 0x3f90, 0x47a3, 0x3f8a,
    0x47d5, 0x3f84, 0x4806, 0x3f7e, 0x4838, 0x3f77, 0x486a, 0x3f71,
    0x489c, 0x3f6a, 0x48ce, 0x3f63, 0x48ff, 0x3f5c, 0x4931, 0x3f55,
    0x4963, 0x3f4e, 0x4995, 0x3f46, 0x49c6, 0x3f3f, 0x49f8, 0x3f37,
    0x4a2a, 0x3f2f, 0x4a5b, 0x3f27, 0x4a8d, 0x3f1f, 0x4abe, 0x3f16,
    0x4af0, 0x3f0e, 0x4b21, 0x3f05, 0x4b53, 0x3efc, 0x4b84, 0x3ef3,
    0x4bb6, 0x3eea, 0x4be7, 0x3ee1, 0x4c19, 0x3ed8, 0x4c4a, 0x3ece,
    0x4c7b, 0x3ec4, 0x4cad, 0x3eba, 0x4cde, 0x3eb0, 0x4d0f, 0x3ea6,
    0x4d40, 0x3e9c, 0x4d71, 0x3e91, 0x4da2, 0x3e87, 0x4dd4, 0x3e7c,
    0x4e05, 0x3e71, 0x4e36, 0x3e66, 0x4e67, 0x3e5b, 0x4e98, 0x3e4f,
    0x4ec9, 0x3e44, 0x4ef9, 0x3e38, 0x4f2a, 0x3e2c, 0x4f5b, 0x3e20,
    0x4f8c, 0x3e14, 0x4fbd, 0x3e08, 0x4fed, 0x3dfb, 0x501e, 0x3def,
    0x504f, 0x3de2, 0x507f, 0x3dd5, 0x50b0, 0x3dc8, 0x50e0, 0x3dbb,
    0x5111, 0x3dae, 0x5141, 0x3da0, 0x5171, 0x3d92, 0x51a2, 0x3d85,
    0x51d2, 0x3d77, 0x5202, 0x3d69, 0x5233, 0x3d5a, 0x5263, 0x3d4c,
    0x5293, 0x3d3e, 0x52c3, 0x3d2f, 0x52f3, 0x3d20, 0x5323, 0x3d11,
    0x5353, 0x3d02, 0x5383, 0x3cf3, 0x53b3, 0x3ce3, 0x53e2, 0x3cd4,
    0x5412, 0x3cc4, 0x5442, 0x3cb4, 0x5472, 0x3ca4, 0x54a1, 0x3c94,
    0x54d1, 0x3c84, 0x5500, 0x3c73, 0x5530, 0x3c63, 0x555f, 0x3c52,
    0x558e, 0x3c41, 0x55be, 0x3c30, 0x55ed, 0x3c1f, 0x561c, 0x3c0e,
    0x564b, 0x3bfc, 0x567a, 0x3beb, 0x56a9, 0x3bd9, 0x56d8, 0x3bc7,
    0x5707, 0x3bb5, 0x5736, 0x3ba3, 0x5765, 0x3b91, 0x5794, 0x3b7e,
    0x57c2, 0x3b6c, 0x57f1, 0x3b59, 0x5820, 0x3b46, 0x584e, 0x3b33,
    0x587d, 0x3b20, 0x58ab, 0x3b0d, 0x58d9, 0x3af9, 0x5908, 0x3ae6,
    0x5936, 0x3ad2, 0x5964, 0x3abe, 0x5992, 0x3aaa, 0x59c0, 0x3a96,
    0x59ee, 0x3a81, 0x5a1c, 0x3a6d, 0x5a4a, 0x3a58, 0x5a78, 0x3a44,
    0x5aa6, 0x3a2f, 0x5ad3, 0x3a1a, 0x5b01, 0x3a05, 0x5b2e, 0x39ef,
    0x5b5c, 0x39da, 0x5b89, 0x39c4, 0x5bb7, 0x39af, 0x5be4, 0x3999,
    0x5c11, 0x3983, 0x5c3e, 0x396d, 0x5c6b, 0x3957, 0x5c98, 0x3940,
    0x5cc5, 0x392a, 0x5cf2, 0x3913, 0x5d1f, 0x38fc, 0x5d4c, 0x38e5,
    0x5d78, 0x38ce, 0x5da5, 0x38b7, 0x5dd1, 0x38a0, 0x5dfe, 0x3888,
    0x5e2a, 0x3870, 0x5e56, 0x3859, 0x5e83, 0x3841, 0x5eaf, 0x3829,
    0x5edb, 0x3811, 0x5f07, 0x37f8, 0x5f33, 0x37e0, 0x5f5f, 0x37c7,
    0x5f8a, 0x37af, 0x5fb6, 0x3796, 0x5fe2, 0x377d, 0x600d, 0x3764,
    0x6039, 0x374a, 0x6064, 0x3731, 0x608f, 0x3718, 0x60bb, 0x36fe,
    0x60e6, 0x36e4, 0x6111, 0x36ca, 0x613c, 0x36b0, 0x6167, 0x3696,
    0x6192, 0x367c, 0x61bc, 0x3661, 0x61e7, 0x3647, 0x6212, 0x362c,
    0x623c, 0x3611, 0x6267, 0x35f6, 0x6291, 0x35db, 0x62bb, 0x35c0,
    0x62e5, 0x35a5, 0x630f, 0x3589, 0x6339, 0x356d, 0x6363, 0x3552,
    0x638d, 0x3536, 0x63b7, 0x351a, 0x63e1, 0x34fe, 0x640a, 0x34e2,
    0x6434, 0x34c5, 0x645d, 0x34a9, 0x6486, 0x348c, 0x64b0, 0x346f,
    0x64d9, 0x3452, 0x6502, 0x3435, 0x652b, 0x3418, 0x6554, 0x33fb,
    0x657c, 0x33de, 0x65a5, 0x33c0, 0x65ce, 0x33a2, 0x65f6, 0x3385,
    0x661f, 0x3367, 0x6647, 0x3349, 0x666f, 0x332b, 0x6697, 0x330c,
    0x66bf, 0x32ee, 0x66e7, 0x32d0, 0x670f, 0x32b1, 0x6737, 0x3292,
    0x675f, 0x3273, 0x6786, 0x3254, 0x67ae, 0x3235, 0x67d5, 0x3216,
    0x67fc, 0x31f7, 0x6824, 0x31d7, 0x684b, 0x31b8, 0x6872, 0x3198,
    0x6899, 0x3178, 0x68bf, 0x3158, 0x68e6, 0x3138, 0x690d, 0x3118,
    0x6933, 0x30f8, 0x695a, 0x30d7, 0x6980, 0x30b7, 0x69a6, 0x3096,
    0x69cc, 0x3075, 0x69f2, 0x3054, 0x6a18, 0x3033, 0x6a3e, 0x3012,
    0x6a64, 0x2ff1, 0x6a89, 0x2fd0, 0x6aaf, 0x2fae, 0x6ad4, 0x2f8d,
    0x6af9, 0x2f6b, 0x6b1f, 0x2f49, 0x6b44, 0x2f27, 0x6b69, 0x2f05,
    0x6b8e, 0x2ee3, 0x6bb2, 0x2ec1, 0x6bd7, 0x2e9e, 0x6bfc, 0x2e7c,
    0x6c20, 0x2e59, 0x6c44, 0x2e36, 0x6c69, 0x2e14, 0x6c8d, 0x2df1,
    0x6cb1, 0x2dce, 0x6cd5, 0x2dab, 0x6cf9, 0x2d87, 0x6d1c, 0x2d64,
    0x6d40, 0x2d40, 0x6d63, 0x2d1d, 0x6d87, 0x2cf9, 0x6daa, 0x2cd5,
    0x6dcd, 0x2cb1, 0x6df0, 0x2c8d, 0x6e13, 0x2c69, 0x6e36, 0x2c45,
    0x6e59, 0x2c21, 0x6e7b, 0x2bfc, 0x6e9e, 0x2bd8, 0x6ec0, 0x2bb3,
    0x6ee2, 0x2b8e, 0x6f05, 0x2b69, 0x6f27, 0x2b44, 0x6f49, 0x2b1f,
    0x6f6a, 0x2afa, 0x6f8c, 0x2ad5, 0x6fae, 0x2aaf, 0x6fcf, 0x2a8a,
    0x6ff0, 0x2a64, 0x7012, 0x2a3e, 0x7033, 0x2a19, 0x7054, 0x29f3,
    0x7075, 0x29cd, 0x7096, 0x29a7, 0x70b6, 0x2980, 0x70d7, 0x295a,
    0x70f7, 0x2934, 0x7117, 0x290d, 0x7138, 0x28e7, 0x7158, 0x28c0,
    0x7178, 0x2899, 0x7197, 0x2872, 0x71b7, 0x284b, 0x71d7, 0x2824,
    0x71f6, 0x27fd, 0x7216, 0x27d6, 0x7235, 0x27ae, 0x7254, 0x2787,
    0x7273, 0x275f, 0x7292, 0x2737, 0x72b0, 0x2710, 0x72cf, 0x26e8,
    0x72ee, 0x26c0, 0x730c, 0x2698, 0x732a, 0x2670, 0x7348, 0x2647,
    0x7366, 0x261f, 0x7384, 0x25f7, 0x73a2, 0x25ce, 0x73c0, 0x25a6,
    0x73dd, 0x257d, 0x73fb, 0x2554, 0x7418, 0x252b, 0x7435, 0x2502,
    0x7452, 0x24d9, 0x746f, 0x24b0, 0x748c, 0x2487, 0x74a8, 0x245e,
    0x74c5, 0x2434, 0x74e1, 0x240b, 0x74fd, 0x23e1, 0x7519, 0x23b7,
    0x7535, 0x238e, 0x7551, 0x2364, 0x756d, 0x233a, 0x7589, 0x2310,
    0x75a4, 0x22e6, 0x75bf, 0x22bc, 0x75db, 0x2291, 0x75f6, 0x2267,
    0x7611, 0x223d, 0x762b, 0x2212, 0x7646, 0x21e8, 0x7661, 0x21bd,
    0x767b, 0x2192, 0x7695, 0x2167, 0x76b0, 0x213c, 0x76ca, 0x2111,
    0x76e4, 0x20e6, 0x76fd, 0x20bb, 0x7717, 0x2090, 0x7731, 0x2065,
    0x774a, 0x2039, 0x7763, 0x200e, 0x777c, 0x1fe2, 0x7795, 0x1fb7,
    0x77ae, 0x1f8b, 0x77c7, 0x1f5f, 0x77df, 0x1f33, 0x77f8, 0x1f07,
    0x7810, 0x1edb, 0x7828, 0x1eaf, 0x7840, 0x1e83, 0x7858, 0x1e57,
    0x7870, 0x1e2b, 0x7888, 0x1dfe, 0x789f, 0x1dd2, 0x78b6, 0x1da5,
    0x78ce, 0x1d79, 0x78e5, 0x1d4c, 0x78fc, 0x1d1f, 0x7912, 0x1cf3,
    0x7929, 0x1cc6, 0x7940, 0x1c99, 0x7956, 0x1c6c, 0x796c, 0x1c3f,
    0x7982, 0x1c12, 0x7998, 0x1be4, 0x79ae, 0x1bb7, 0x79c4, 0x1b8a,
    0x79da, 0x1b5c, 0x79ef, 0x1b2f, 0x7a04, 0x1b01, 0x7a19, 0x1ad4,
    0x7a2e, 0x1aa6, 0x7a43, 0x1a78, 0x7a58, 0x1a4b, 0x7a6d, 0x1a1d,
    0x7a81, 0x19ef, 0x7a95, 0x19c1, 0x7aa9, 0x1993, 0x7abd, 0x1965,
    0x7ad1, 0x1936, 0x7ae5, 0x1908, 0x7af9, 0x18da, 0x7b0c, 0x18ac,
    0x7b1f, 0x187d, 0x7b33, 0x184f, 0x7b46, 0x1820, 0x7b58, 0x17f2,
    0x7b6b, 0x17c3, 0x7b7e, 0x1794, 0x7b90, 0x1766, 0x7ba3, 0x1737,
    0x7bb5, 0x1708, 0x7bc7, 0x16d9, 0x7bd9, 0x16aa, 0x7bea, 0x167b,
    0x7bfc, 0x164c, 0x7c0d, 0x161d, 0x7c1f, 0x15ed, 0x7c30, 0x15be,
    0x7c41, 0x158f, 0x7c52, 0x1560, 0x7c62, 0x1530, 0x7c73, 0x1501,
    0x7c83, 0x14d1, 0x7c94, 0x14a2, 0x7ca4, 0x1472, 0x7cb4, 0x1442,
    0x7cc4, 0x1413, 0x7cd3, 0x13e3, 0x7ce3, 0x13b3, 0x7cf2, 0x1383,
    0x7d01, 0x1353, 0x7d11, 0x1323, 0x7d20, 0x12f3, 0x7d2e, 0x12c3,
    0x7d3d, 0x1293, 0x7d4c, 0x1263, 0x7d5a, 0x1233, 0x7d68, 0x1203,
    0x7d76, 0x11d3, 0x7d84, 0x11a2, 0x7d92, 0x1172, 0x7da0, 0x1142,
    0x7dad, 0x1111, 0x7dba, 0x10e1, 0x7dc8, 0x10b0, 0x7dd5, 0x1080,
    0x7de1, 0x104f, 0x7dee, 0x101e, 0x7dfb, 0x0fee, 0x7e07, 0x0fbd,
    0x7e14, 0x0f8c, 0x7e20, 0x0f5c, 0x7e2c, 0x0f2b, 0x7e37, 0x0efa,
    0x7e43, 0x0ec9, 0x7e4f, 0x0e98, 0x7e5a, 0x0e67, 0x7e65, 0x0e36,
    0x7e70, 0x0e05, 0x7e7b, 0x0dd4, 0x7e86, 0x0da3, 0x7e91, 0x0d72,
    0x7e9b, 0x0d41, 0x7ea6, 0x0d10, 0x7eb0, 0x0cde, 0x7eba, 0x0cad,
    0x7ec4, 0x0c7c, 0x7ecd, 0x0c4a, 0x7ed7, 0x0c19, 0x7ee0, 0x0be8,
    0x7eea, 0x0bb6, 0x7ef3, 0x0b85, 0x7efc, 0x0b53, 0x7f05, 0x0b22,
    0x7f0d, 0x0af0, 0x7f16, 0x0abf, 0x7f1e, 0x0a8d, 0x7f26, 0x0a5c,
    0x7f2e, 0x0a2a, 0x7f36, 0x09f9, 0x7f3e, 0x09c7, 0x7f46, 0x0995,
    0x7f4d, 0x0963, 0x7f54, 0x0932, 0x7f5c, 0x0900, 0x7f63, 0x08ce,
    0x7f69, 0x089c, 0x7f70, 0x086b, 0x7f77, 0x0839, 0x7f7d, 0x0807,
    0x7f83, 0x07d5, 0x7f89, 0x07a3, 0x7f8f, 0x0771, 0x7f95, 0x073f,
    0x7f9b, 0x070d, 0x7fa0, 0x06db, 0x7fa5, 0x06a9, 0x7fab, 0x0677,
    0x7fb0, 0x0645, 0x7fb4, 0x0613, 0x7fb9, 0x05e1, 0x7fbe, 0x05af,
    0x7fc2, 0x057d, 0x7fc6, 0x054b, 0x7fca, 0x0519, 0x7fce, 0x04e7,
    0x7fd2, 0x04b5, 0x7fd6, 0x0483, 0x7fd9, 0x0450, 0x7fdd, 0x041e,
    0x7fe0, 0x03ec, 0x7fe3, 0x03ba, 0x7fe6, 0x0388, 0x7fe8, 0x0356,
    0x7feb, 0x0323, 0x7fed, 0x02f1, 0x7fef, 0x02bf, 0x7ff1, 0x028d,
    0x7ff3, 0x025b, 0x7ff5, 0x0228, 0x7ff7, 0x01f6, 0x7ff8, 0x01c4,
    0x7ffa, 0x0192, 0x7ffb, 0x015f, 0x7ffc, 0x012d, 0x7ffd, 0x00fb,
    0x7ffd, 0x00c9, 0x7ffe, 0x0096, 0x7ffe, 0x0064, 0x7ffe, 0x0032,
    0x7fff, 0x0000, 0x7ffe, 0xffce, 0x7ffe, 0xff9c, 0x7ffe, 0xff6a,
    0x7ffd, 0xff37, 0x7ffd, 0xff05, 0x7ffc, 0xfed3, 0x7ffb, 0xfea1,
    0x7ffa, 0xfe6e, 0x7ff8, 0xfe3c, 0x7ff7, 0xfe0a, 0x7ff5, 0xfdd8,
    0x7ff3, 0xfda5, 0x7ff1, 0xfd73, 0x7fef, 0xfd41, 0x7fed, 0xfd0f,
    0x7feb, 0xfcdd, 0x7fe8, 0xfcaa, 0x7fe6, 0xfc78, 0x7fe3, 0xfc46,
    0x7fe0, 0xfc14, 0x7fdd, 0xfbe2, 0x7fd9, 0xfbb0, 0x7fd6, 0xfb7d,
    0x7fd2, 0xfb4b, 0x7fce, 0xfb19, 0x7fca, 0xfae7, 0x7fc6, 0xfab5,
    0x7fc2, 0xfa83, 0x7fbe, 0xfa51, 0x7fb9, 0xfa1f, 0x7fb4, 0xf9ed,
    0x7fb0, 0xf9bb, 0x7fab, 0xf989, 0x7fa5, 0xf957, 0x7fa0, 0xf925,
    0x7f9b, 0xf8f3, 0x7f95, 0xf8c1, 0x7f8f, 0xf88f, 0x7f89, 0xf85d,
    0x7f83, 0xf82b, 0x7f7d, 0xf7f9, 0x7f77, 0xf7c7, 0x7f70, 0xf795,
    0x7f69, 0xf764, 0x7f63, 0xf732, 0x7f5c, 0xf700, 0x7f54, 0xf6ce,
    0x7f4d, 0xf69d, 0x7f46, 0xf66b, 0x7f3e, 0xf639, 0x7f36, 0xf607,
    0x7f2e, 0xf5d6, 0x7f26, 0xf5a4, 0x7f1e, 0xf573, 0x7f16, 0xf541,
    0x7f0d, 0xf510, 0x7f05, 0xf4de, 0x7efc, 0xf4ad, 0x7ef3, 0xf47b,
    0x7eea, 0xf44a, 0x7ee0, 0xf418, 0x7ed7, 0xf3e7, 0x7ecd, 0xf3b6,
    0x7ec4, 0xf384, 0x7eba, 0xf353, 0x7eb0, 0xf322, 0x7ea6, 0xf2f0,
    0x7e9b, 0xf2bf, 0x7e91, 0xf28e, 0x7e86, 0xf25d, 0x7e7b, 0xf22c,
    0x7e70, 0xf1fb, 0x7e65, 0xf1ca, 0x7e5a, 0xf199, 0x7e4f, 0xf168,
    0x7e43, 0xf137, 0x7e37, 0xf106, 0x7e2c, 0xf0d5, 0x7e20, 0xf0a4,
    0x7e14, 0xf074, 0x7e07, 0xf043, 0x7dfb, 0xf012, 0x7dee, 0xefe2,
    0x7de1, 0xefb1, 0x7dd5, 0xef80, 0x7dc8, 0xef50, 0x7dba, 0xef1f,
    0x7dad, 0xeeef, 0x7da0, 0xeebe, 0x7d92, 0xee8e, 0x7d84, 0xee5e,
    0x7d76, 0xee2d, 0x7d68, 0xedfd, 0x7d5a, 0xedcd, 0x7d4c, 0xed9d,
    0x7d3d, 0xed6d, 0x7d2e, 0xed3d, 0x7d20, 0xed0d, 0x7d11, 0xecdd,
    0x7d01, 0xecad, 0x7cf2, 0xec7d, 0x7ce3, 0xec4d, 0x7cd3, 0xec1d,
    0x7cc4, 0xebed, 0x7cb4, 0xebbe, 0x7ca4, 0xeb8e, 0x7c94, 0xeb5e,
    0x7c83, 0xeb2f, 0x7c73, 0xeaff, 0x7c62, 0xead0, 0x7c52, 0xeaa0,
    0x7c41, 0xea71, 0x7c30, 0xea42, 0x7c1f, 0xea13, 0x7c0d, 0xe9e3,
    0x7bfc, 0xe9b4, 0x7bea, 0xe985, 0x7bd9, 0xe956, 0x7bc7, 0xe927,
    0x7bb5, 0xe8f8, 0x7ba3, 0xe8c9, 0x7b90, 0xe89a, 0x7b7e, 0xe86c,
    0x7b6b, 0xe83d, 0x7b58, 0xe80e, 0x7b46, 0xe7e0, 0x7b33, 0xe7b1,
    0x7b1f, 0xe783, 0x7b0c, 0xe754, 0x7af9, 0xe726, 0x7ae5, 0xe6f8,
    0x7ad1, 0xe6ca, 0x7abd, 0xe69b, 0x7aa9, 0xe66d, 0x7a95, 0xe63f,
    0x7a81, 0xe611, 0x7a6d, 0xe5e3, 0x7a58, 0xe5b5, 0x7a43, 0xe588,
    0x7a2e, 0xe55a, 0x7a19, 0xe52c, 0x7a04, 0xe4ff, 0x79ef, 0xe4d1,
    0x79da, 0xe4a4, 0x79c4, 0xe476, 0x79ae, 0xe449, 0x7998, 0xe41c,
    0x7982, 0xe3ee, 0x796c, 0xe3c1, 0x7956, 0xe394, 0x7940, 0xe367,
    0x7929, 0xe33a, 0x7912, 0xe30d, 0x78fc, 0xe2e1, 0x78e5, 0xe2b4,
    0x78ce, 0xe287, 0x78b6, 0xe25b, 0x789f, 0xe22e, 0x7888, 0xe202,
    0x7870, 0xe1d5, 0x7858, 0xe1a9, 0x7840, 0xe17d, 0x7828, 0xe151,
    0x7810, 0xe125, 0x77f8, 0xe0f9, 0x77df, 0xe0cd, 0x77c7, 0xe0a1,
    0x77ae, 0xe075, 0x7795, 0xe049, 0x777c, 0xe01e, 0x7763, 0xdff2,
    0x774a, 0xdfc7, 0x7731, 0xdf9b, 0x7717, 0xdf70, 0x76fd, 0xdf45,
    0x76e4, 0xdf1a, 0x76ca, 0xdeef, 0x76b0, 0xdec4, 0x7695, 0xde99,
    0x767b, 0xde6e, 0x7661, 0xde43, 0x7646, 0xde18, 0x762b, 0xddee,
    0x7611, 0xddc3, 0x75f6, 0xdd99, 0x75db, 0xdd6f, 0x75bf, 0xdd44,
    0x75a4, 0xdd1a, 0x7589, 0xdcf0, 0x756d, 0xdcc6, 0x7551, 0xdc9c,
    0x7535, 0xdc72, 0x7519, 0xdc49, 0x74fd, 0xdc1f, 0x74e1, 0xdbf5,
    0x74c5, 0xdbcc, 0x74a8, 0xdba2, 0x748c, 0xdb79, 0x746f, 0xdb50,
    0x7452, 0xdb27, 0x7435, 0xdafe, 0x7418, 0xdad5, 0x73fb, 0xdaac,
    0x73dd, 0xda83, 0x73c0, 0xda5a, 0x73a2, 0xda32, 0x7384, 0xda09,
    0x7366, 0xd9e1, 0x7348, 0xd9b9, 0x732a, 0xd990, 0x730c, 0xd968,
    0x72ee, 0xd940, 0x72cf, 0xd918, 0x72b0, 0xd8f0, 0x7292, 0xd8c9,
    0x7273, 0xd8a1, 0x7254, 0xd879, 0x7235, 0xd852, 0x7216, 0xd82a,
    0x71f6, 0xd803, 0x71d7, 0xd7dc, 0x71b7, 0xd7b5, 0x7197, 0xd78e,
    0x7178, 0xd767, 0x7158, 0xd740, 0x7138, 0xd719, 0x7117, 0xd6f3,
    0x70f7, 0xd6cc, 0x70d7, 0xd6a6, 0x70b6, 0xd680, 0x7096, 0xd659,
    0x7075, 0xd633, 0x7054, 0xd60d, 0x7033, 0xd5e7, 0x7012, 0xd5c2,
    0x6ff0, 0xd59c, 0x6fcf, 0xd576, 0x6fae, 0xd551, 0x6f8c, 0xd52b,
    0x6f6a, 0xd506, 0x6f49, 0xd4e1, 0x6f27, 0xd4bc, 0x6f05, 0xd497,
    0x6ee2, 0xd472, 0x6ec0, 0xd44d, 0x6e9e, 0xd428, 0x6e7b, 0xd404,
    0x6e59, 0xd3df, 0x6e36, 0xd3bb, 0x6e13, 0xd397, 0x6df0, 0xd373,
    0x6dcd, 0xd34f, 0x6daa, 0xd32b, 0x6d87, 0xd307, 0x6d63, 0xd2e3,
    0x6d40, 0xd2c0, 0x6d1c, 0xd29c, 0x6cf9, 0xd279, 0x6cd5, 0xd255,
    0x6cb1, 0xd232, 0x6c8d, 0xd20f, 0x6c69, 0xd1ec, 0x6c44, 0xd1ca,
    0x6c20, 0xd1a7, 0x6bfc, 0xd184, 0x6bd7, 0xd162, 0x6bb2, 0xd13f,
    0x6b8e, 0xd11d, 0x6b69, 0xd0fb, 0x6b44, 0xd0d9, 0x6b1f, 0xd0b7,
    0x6af9, 0xd095, 0x6ad4, 0xd073, 0x6aaf, 0xd052, 0x6a89, 0xd030,
    0x6a64, 0xd00f, 0x6a3e, 0xcfee, 0x6a18, 0xcfcd, 0x69f2, 0xcfac,
    0x69cc, 0xcf8b, 0x69a6, 0xcf6a, 0x6980, 0xcf49, 0x695a, 0xcf29,
    0x6933, 0xcf08, 0x690d, 0xcee8, 0x68e6, 0xcec8, 0x68bf, 0xcea8,
    0x6899, 0xce88, 0x6872, 0xce68, 0x684b, 0xce48, 0x6824, 0xce29,
    0x67fc, 0xce09, 0x67d5, 0xcdea, 0x67ae, 0xcdcb, 0x6786, 0xcdac,
    0x675f, 0xcd8d, 0x6737, 0xcd6e, 0x670f, 0xcd4f, 0x66e7, 0xcd30,
    0x66bf, 0xcd12, 0x6697, 0xccf4, 0x666f, 0xccd5, 0x6647, 0xccb7,
    0x661f, 0xcc99, 0x65f6, 0xcc7b, 0x65ce, 0xcc5e, 0x65a5, 0xcc40,
    0x657c, 0xcc22, 0x6554, 0xcc05, 0x652b, 0xcbe8, 0x6502, 0xcbcb,
    0x64d9, 0xcbae, 0x64b0, 0xcb91, 0x6486, 0xcb74, 0x645d, 0xcb57,
    0x6434, 0xcb3b, 0x640a, 0xcb1e, 0x63e1, 0xcb02, 0x63b7, 0xcae6,
    0x638d, 0xcaca, 0x6363, 0xcaae, 0x6339, 0xca93, 0x630f, 0xca77,
    0x62e5, 0xca5b, 0x62bb, 0xca40, 0x6291, 0xca25, 0x6267, 0xca0a,
    0x623c, 0xc9ef, 0x6212, 0xc9d4, 0x61e7, 0xc9b9, 0x61bc, 0xc99f,
    0x6192, 0xc984, 0x6167, 0xc96a, 0x613c, 0xc950, 0x6111, 0xc936,
    0x60e6, 0xc91c, 0x60bb, 0xc902, 0x608f, 0xc8e8, 0x6064, 0xc8cf,
    0x6039, 0xc8b6, 0x600d, 0xc89c, 0x5fe2, 0xc883, 0x5fb6, 0xc86a,
    0x5f8a, 0xc851, 0x5f5f, 0xc839, 0x5f33, 0xc820, 0x5f07, 0xc808,
    0x5edb, 0xc7ef, 0x5eaf, 0xc7d7, 0x5e83, 0xc7bf, 0x5e56, 0xc7a7,
    0x5e2a, 0xc790, 0x5dfe, 0xc778, 0x5dd1, 0xc760, 0x5da5, 0xc749,
    0x5d78, 0xc732, 0x5d4c, 0xc71b, 0x5d1f, 0xc704, 0x5cf2, 0xc6ed,
    0x5cc5, 0xc6d6, 0x5c98, 0xc6c0, 0x5c6b, 0xc6a9, 0x5c3e, 0xc693,
    0x5c11, 0xc67d, 0x5be4, 0xc667, 0x5bb7, 0xc651, 0x5b89, 0xc63c,
    0x5b5c, 0xc626, 0x5b2e, 0xc611, 0x5b01, 0xc5fb, 0x5ad3, 0xc5e6,
    0x5aa6, 0xc5d1, 0x5a78, 0xc5bc, 0x5a4a, 0xc5a8, 0x5a1c, 0xc593,
    0x59ee, 0xc57f, 0x59c0, 0xc56a, 0x5992, 0xc556, 0x5964, 0xc542,
    0x5936, 0xc52e, 0x5908, 0xc51a, 0x58d9, 0xc507, 0x58ab, 0xc4f3,
    0x587d, 0xc4e0, 0x584e, 0xc4cd, 0x5820, 0xc4ba, 0x57f1, 0xc4a7,
    0x57c2, 0xc494, 0x5794, 0xc482, 0x5765, 0xc46f, 0x5736, 0xc45d,
    0x5707, 0xc44b, 0x56d8, 0xc439, 0x56a9, 0xc427, 0x567a, 0xc415,
    0x564b, 0xc404, 0x561c, 0xc3f2, 0x55ed, 0xc3e1, 0x55be, 0xc3d0,
    0x558e, 0xc3bf, 0x555f, 0xc3ae, 0x5530, 0xc39d, 0x5500, 0xc38d,
    0x54d1, 0xc37c, 0x54a1, 0xc36c, 0x5472, 0xc35c, 0x5442, 0xc34c,
    0x5412, 0xc33c, 0x53e2, 0xc32c, 0x53b3, 0xc31d, 0x5383, 0xc30d,
    0x5353, 0xc2fe, 0x5323, 0xc2ef, 0x52f3, 0xc2e0, 0x52c3, 0xc2d1,
    0x5293, 0xc2c2, 0x5263, 0xc2b4, 0x5233, 0xc2a6, 0x5202, 0xc297,
    0x51d2, 0xc289, 0x51a2, 0xc27b, 0x5171, 0xc26e, 0x5141, 0xc260,
    0x5111, 0xc252, 0x50e0, 0xc245, 0x50b0, 0xc238, 0x507f, 0xc22b,
    0x504f, 0xc21e, 0x501e, 0xc211, 0x4fed, 0xc205, 0x4fbd, 0xc1f8,
    0x4f8c, 0xc1ec, 0x4f5b, 0xc1e0, 0x4f2a, 0xc1d4, 0x4ef9, 0xc1c8,
    0x4ec9, 0xc1bc, 0x4e98, 0xc1b1, 0x4e67, 0xc1a5, 0x4e36, 0xc19a,
    0x4e05, 0xc18f, 0x4dd4, 0xc184, 0x4da2, 0xc179, 0x4d71, 0xc16f,
    0x4d40, 0xc164, 0x4d0f, 0xc15a, 0x4cde, 0xc150, 0x4cad, 0xc146,
    0x4c7b, 0xc13c, 0x4c4a, 0xc132, 0x4c19, 0xc128, 0x4be7, 0xc11f,
    0x4bb6, 0xc116, 0x4b84, 0xc10d, 0x4b53, 0xc104, 0x4b21, 0xc0fb,
    0x4af0, 0xc0f2, 0x4abe, 0xc0ea, 0x4a8d, 0xc0e1, 0x4a5b, 0xc0d9,
    0x4a2a, 0xc0d1, 0x49f8, 0xc0c9, 0x49c6, 0xc0c1, 0x4995, 0xc0ba,
    0x4963, 0xc0b2, 0x4931, 0xc0ab, 0x48ff, 0xc0a4, 0x48ce, 0xc09d,
    0x489c, 0xc096, 0x486a, 0xc08f, 0x4838, 0xc089, 0x4806, 0xc082,
    0x47d5, 0xc07c, 0x47a3, 0xc076, 0x4771, 0xc070, 0x473f, 0xc06a,
    0x470d, 0xc065, 0x46db, 0xc05f, 0x46a9, 0xc05a, 0x4677, 0xc055,
    0x4645, 0xc050, 0x4613, 0xc04b, 0x45e1, 0xc046, 0x45af, 0xc042,
    0x457d, 0xc03d, 0x454b, 0xc039, 0x4518, 0xc035, 0x44e6, 0xc031,
    0x44b4, 0xc02d, 0x4482, 0xc02a, 0x4450, 0xc026, 0x441e, 0xc023,
    0x43ec, 0xc020, 0x43b9, 0xc01d, 0x4387, 0xc01a, 0x4355, 0xc017,
    0x4323, 0xc015, 0x42f1, 0xc012, 0x42be, 0xc010, 0x428c, 0xc00e,
    0x425a, 0xc00c, 0x4228, 0xc00a, 0x41f6, 0xc009, 0x41c3, 0xc007,
    0x4191, 0xc006, 0x415f, 0xc005, 0x412d, 0xc004, 0x40fa, 0xc003,
    0x40c8, 0xc002, 0x4096, 0xc002, 0x4064, 0xc001, 0x4031, 0xc001,
    };

// Pseudo code for generation of the bit-reversal-table...
// for (k=1; k<=FFT_N/4; k++)
// {
//     for (i=0; i<log2(FFT_N); i++)
//     {
//         a[i] = k & (1 << i);
//     }
//     for (j=0; j<log2(FFT_N); j++)
//     {
//         if (a[j] != 0)
//             y[k] += (1 << ((log2(FFT_N)-1)-j));
//     }
//     y[k] = y[k] >> 1;
// }
// FFT_N=256 (number of FFT-bins) ==> log2(FFT_N) = 8

// Table for bit-reversal process for 1024-bin FFT
static const uint16_t reverse_table[256] = {
    0x0100, 0x0080, 0x0180, 0x0040, 0x0140, 0x00c0, 0x01c0, 0x0020, 
    0x0120, 0x00a0, 0x01a0, 0x0060, 0x0160, 0x00e0, 0x01e0, 0x0010, 
    0x0110, 0x0090, 0x0190, 0x0050, 0x0150, 0x00d0, 0x01d0, 0x0030, 
    0x0130, 0x00b0, 0x01b0, 0x0070, 0x0170, 0x00f0, 0x01f0, 0x0008, 
    0x0108, 0x0088, 0x0188, 0x0048, 0x0148, 0x00c8, 0x01c8, 0x0028, 
    0x0128, 0x00a8, 0x01a8, 0x0068, 0x0168, 0x00e8, 0x01e8, 0x0018, 
    0x0118, 0x0098, 0x0198, 0x0058, 0x0158, 0x00d8, 0x01d8, 0x0038, 
    0x0138, 0x00b8, 0x01b8, 0x0078, 0x0178, 0x00f8, 0x01f8, 0x0004, 
    0x0104, 0x0084, 0x0184, 0x0044, 0x0144, 0x00c4, 0x01c4, 0x0024, 
    0x0124, 0x00a4, 0x01a4, 0x0064, 0x0164, 0x00e4, 0x01e4, 0x0014, 
    0x0114, 0x0094, 0x0194, 0x0054, 0x0154, 0x00d4, 0x01d4, 0x0034, 
    0x0134, 0x00b4, 0x01b4, 0x0074, 0x0174, 0x00f4, 0x01f4, 0x000c, 
    0x010c, 0x008c, 0x018c, 0x004c, 0x014c, 0x00cc, 0x01cc, 0x002c, 
    0x012c, 0x00ac, 0x01ac, 0x006c, 0x016c, 0x00ec, 0x01ec, 0x001c, 
    0x011c, 0x009c, 0x019c, 0x005c, 0x015c, 0x00dc, 0x01dc, 0x003c, 
    0x013c, 0x00bc, 0x01bc, 0x007c, 0x017c, 0x00fc, 0x01fc, 0x0002, 
    0x0102, 0x0082, 0x0182, 0x0042, 0x0142, 0x00c2, 0x01c2, 0x0022, 
    0x0122, 0x00a2, 0x01a2, 0x0062, 0x0162, 0x00e2, 0x01e2, 0x0012, 
    0x0112, 0x0092, 0x0192, 0x0052, 0x0152, 0x00d2, 0x01d2, 0x0032, 
    0x0132, 0x00b2, 0x01b2, 0x0072, 0x0172, 0x00f2, 0x01f2, 0x000a, 
    0x010a, 0x008a, 0x018a, 0x004a, 0x014a, 0x00ca, 0x01ca, 0x002a, 
    0x012a, 0x00aa, 0x01aa, 0x006a, 0x016a, 0x00ea, 0x01ea, 0x001a, 
    0x011a, 0x009a, 0x019a, 0x005a, 0x015a, 0x00da, 0x01da, 0x003a, 
    0x013a, 0x00ba, 0x01ba, 0x007a, 0x017a, 0x00fa, 0x01fa, 0x0006, 
    0x0106, 0x0086, 0x0186, 0x0046, 0x0146, 0x00c6, 0x01c6, 0x0026, 
    0x0126, 0x00a6, 0x01a6, 0x0066, 0x0166, 0x00e6, 0x01e6, 0x0016, 
    0x0116, 0x0096, 0x0196, 0x0056, 0x0156, 0x00d6, 0x01d6, 0x0036, 
    0x0136, 0x00b6, 0x01b6, 0x0076, 0x0176, 0x00f6, 0x01f6, 0x000e, 
    0x010e, 0x008e, 0x018e, 0x004e, 0x014e, 0x00ce, 0x01ce, 0x002e, 
    0x012e, 0x00ae, 0x01ae, 0x006e, 0x016e, 0x00ee, 0x01ee, 0x001e, 
    0x011e, 0x009e, 0x019e, 0x005e, 0x015e, 0x00de, 0x01de, 0x003e, 
    0x013e, 0x00be, 0x01be, 0x007e, 0x017e, 0x00fe, 0x01fe, 0x0001, 
    };

/*!
 *****************************************************************************
 * \brief In-place bit reversal function.
 *
 * \param *data_in - complex input vector (processing occurs in-place)
 * \param len - length of the FFT.
 *****************************************************************************
 */
#define MAX_FFT_LEN 1024
#define MODIFIER(x) (MAX_FFT_LEN/x)
static void bit_reverse(int16_t *data_in, uint16_t len)
{   //Must be performed after a radix transformation
    int32_t *data = (int32_t *)data_in;
    int32_t tmp;
    uint16_t half_len;
    uint16_t i, j;
    uint16_t table_index;

    // Initialization
    j = 0;
    half_len = len / 2;
    table_index = (MODIFIER(len) - 1);
    for (i=0; i < half_len; i+=2)
    {
        if (i < j)
        {
            // data[i] <-> data[j];
            tmp = data[i];
            data[i] = data[j];
            data[j] = tmp;

            // data[i + half_len + 1] <-> data[j + half_len + 1];
            tmp = data[i + half_len + 1];
            data[i + half_len + 1] = data[j + half_len + 1];
            data[j + half_len + 1] = tmp;
        }
        // data[i+1] <-> data[j+half_len];
        tmp = data[i + 1];
        data[i + 1] = data[j + half_len];
        data[j + half_len] = tmp;
        // Reading the index for the bit reversal
        j = reverse_table[table_index];
        // Updating the bit-reversal index depending on the fft length
        table_index += MODIFIER(len);
    }
}

/*!
 *****************************************************************************
 * \brief Inline 16-bit saturation
 *
 * \param x - the 32-bit value to saturate to 16-bits
 * \returns - the 16-bit saturated result
 *****************************************************************************
 */
/* __STATIC_INLINE */ int32_t __SSAT_16(int32_t x)
{
    if (x > 32767)
    {
        x = 32767;
    }
    else if (x < -32768)
    {
        x = -32768;
    }
    return x;
}

// Radix-4 FFT algorithm...
//
// Input real and imaginary data:
//   x(n) = xa + j * ya
//   x(n+N/4) = xb + j * yb
//   x(n+N/2) = xc + j * yc
//   x(n+3N/4) = xd + j * yd
//
// Output real and imaginary data:
//   x(4r) = xa'+ j * ya'
//   x(4r+1) = xb'+ j * yb'
//   x(4r+2) = xc'+ j * yc'
//   x(4r+3) = xd'+ j * yd'
//
// Twiddle factors for radix-4 FFT:
//   Wn = cos1 + j * (-sin1)
//   W2n = cos2 + j * (-sin2)
//   W3n = cos3 + j * (-sin3)
//
// The real and imaginary output values for the radix-4 butterfly are
//   xa' = xa + xb + xc + xd
//   ya' = ya + yb + yc + yd
//   xb' = (xa+yb-xc-yd)*cos1 + (ya-xb-yc+xd)*sin1
//   yb' = (ya-xb-yc+xd)*cos1 - (xa+yb-xc-yd)*sin1
//   xc' = (xa-xb+xc-xd)*cos2 + (ya-yb+yc-yd)*sin2
//   yc' = (ya-yb+yc-yd)*cos2 - (xa-xb+xc-xd)*sin2
//   xd' = (xa-yb-xc+yd)*cos3 + (ya+xb-yc-xd)*sin3
//   yd' = (ya+xb-yc-xd)*cos3 - (xa-yb-xc+yd)*sin3
//

/*!
 *****************************************************************************
 * \brief Core function for the 16-bit CFFT butterfly process.
 * Internally the input is downscaled by 2 for every stage to avoid saturations
 * during CFFT processing.
 *
 * \param *data - complex input vector (processing occurs in-place)
 * \param len - length of the FFT.
 *****************************************************************************
 */
static void radix4_butterfly(int16_t *data, uint16_t len)
{   //Heavily optimized, Non-optimized (but cleaner) code is above
    //@TODO: Some of these comments are inconsistent with code

    uint16_t modifier;
    modifier = MODIFIER(len);
    int16_t R0, R1, S0, S1, T0, T1, U0, U1;
    int16_t cos1, sin1, cos2, sin2, cos3, sin3, out1, out2;
    uint32_t n1, n2, ic, i0, i1, i2, i3, j, k;

    // Total processing is divided into three stages

    // Initialization for the first stage
    n2 = len;
    n1 = n2;

    // n2 = len/4
    n2 >>= 2;

    // Index for twiddle coefficient
    ic = 0;

    // Index for input read and output write
    i0 = 0;
    j = n2;

    // Input is in 1.15(q15) format

    // First stage processing...
    do
    {
        /* Butterfly implementation */

        /* index calculation for the inputs... */
        /* data[i0 + 0], pSrc16[i0 + len/4], pSrc16[i0 + fftLen/2], pSrc16[i0 + 3fftLen/4] */
        i1 = i0 + n2;
        i2 = i1 + n2;
        i3 = i2 + n2;

        /* Reading i0, i0+len/2 inputs */

        /* input is down scale by 4 to avoid overflow */
        /* Read ya(real), xa(imag) input */
        T0 = data[i0 * 2] >> 2;  // --1.13
        T1 = data[(i0 * 2) + 1] >> 2;  // --1.13

        /* input is down scale by 4 to avoid overflow */
        /* Read yc(real), xc(imag) input */
        S0 = data[i2 * 2] >> 2;  // --1.13
        S1 = data[(i2 * 2) + 1] >> 2;  // --1.13

        /* R0 = (ya + yc), R1 = (xa + xc) */
        R0 = (T0 + S0);  // -2.13
        R1 = (T1 + S1);  // -2.13

        /* S0 = (ya - yc), S1 = (xa - xc) */
        S0 = (T0 - S0);  // -2.13
        S1 = (T1 - S1);  // -2.13

        /* Reading i0+len/4, i0+3fftLen/4 inputs */
        /* input is down scale by 4 to avoid overflow */
        /* Read yb(real), xb(imag) input */
        T0 = data[i1 * 2] >> 2;  // --1.13
        T1 = data[(i1 * 2) + 1] >> 2;  // --1.13

        /* input is down scale by 4 to avoid overflow */
        /* Read yd(real), xd(imag) input */
        U0 = data[i3 * 2] >> 2;  // --1.13
        U1 = data[(i3 * 2) + 1] >> 2;  // --1.13

        /* T0 = (yb + yd), T1 = (xb + xd) */
        T0 = (T0 + U0);  // -2.13
        T1 = (T1 + U1);  // -2.13

        /* writing the butterfly processed i0 sample */
        /* ya' = ya + yb + yc + yd */
        /* xa' = xa + xb + xc + xd */
        data[i0 * 2] = (R0 >> 1) + (T0 >> 1); // (--2.12) + (--2.12) ==> (-3.12)
        data[(i0 * 2) + 1] = (R1 >> 1) + (T1 >> 1); // (--2.12) + (--2.12) ==> (-3.12)

        /* R0 = (ya + yc) - (yb + yd) */
        /* R1 = (xa + xc) - (xb + xd) */
        R0 = (R0 - T0);  // 3.13
        R1 = (R1 - T1);  // 3.13

        /* cos2 & sin2 are read from Coefficient pointer */
        cos2 = twiddles[2 * (ic * 2)]; // 1.15
        sin2 = twiddles[(2 * (ic * 2)) + 1]; // 1.15

        /* xc' = (xa-xb+xc-xd) * cos2 + (ya-yb+yc-yd) * sin2 */
        /* yc' = (ya-yb+yc-yd) * cos2 - (xa-xb+xc-xd) * sin2 */
        out1 = (int16_t)((cos2 * R0 + sin2 * R1) >> 16); // (-3.28) + (-3.28) = 4.28 ==> 4.12
        out2 = (int16_t)((-sin2 * R0 + cos2 * R1) >> 16); // (-3.28) + (-3.28) = 4.28 ==> 4.12

        /* Reading i0+len/4 */
        /* input is down scale by 4 to avoid overflow */
        /* T0 = yb, T1 = xb */
        T0 = data[i1 * 2] >> 2;  // --1.13
        T1 = data[(i1 * 2) + 1] >> 2;  // --1.13

        /* writing the butterfly processed i0 + len/4 sample */
        /* writing output(xc', yc') in little endian cos1format */
        data[i1 * 2] = out1;  // 4.12
        data[(i1 * 2) + 1] = out2;  // 4.12

        /* Butterfly calculations */
        /* input is down scale by 4 to avoid overflow */
        /* U0 = yd, U1 = xd */
        U0 = data[i3 * 2] >> 2;  // --1.13
        U1 = data[(i3 * 2) + 1] >> 2;  // --1.13

        /* T0 = yb-yd, T1 = xb-xd */
        T0 = (T0 - U0);  // -2.13
        T1 = (T1 - U1);  // -2.13

        /* R1 = (ya-yc) + (xb- xd),  R0 = (xa-xc) - (yb-yd) */
        R0 = (S0 - T1);  // 3.13
        R1 = (S1 + T0);  // 3.13

        /* S1 = (ya-yc) - (xb- xd), S0 = (xa-xc) + (yb-yd) */
        S0 = (S0 + T1);  // 3.13
        S1 = (S1 - T0);  // 3.13

        /* cos1 & sin1 are read from Coefficient pointer */
        cos1 = twiddles[ic * 2]; // 1.15
        sin1 = twiddles[(ic * 2) + 1]; // 1.15

        /* Butterfly process for the i0+len/2 sample */
        /* xb' = (xa+yb-xc-yd) * cos1 + (ya-xb-yc+xd) * sin1 */
        /* yb' = (ya-xb-yc+xd) * cos1 - (xa+yb-xc-yd) * sin1 */
        out1 = (int16_t)((sin1 * S1 + cos1 * S0) >> 16); // (-3.28) + (-3.28) = 4.28 ==> 4.12
        out2 = (int16_t)((-sin1 * S0 + cos1 * S1) >> 16); // (-3.28) + (-3.28) = 4.28 ==> 4.12
        /* writing output(xb', yb') in little endian format */
        data[i2 * 2] = out1;  // 4.12
        data[(i2 * 2) + 1] = out2;  // 4.12

        /* cos3 & sin3 are read from Coefficient pointer */
        cos3 = twiddles[3 * (ic * 2)]; // 1.15
        sin3 = twiddles[(3 * (ic * 2)) + 1]; // 1.15

        /* Butterfly process for the i0+3len/4 sample */
        /* xd' = (xa-yb-xc+yd) * cos3 + (ya+xb-yc-xd) * sin3 */
        /* yd' = (ya+xb-yc-xd) * cos3 - (xa-yb-xc+yd) * sin3 */
        out1 = (int16_t)((sin3 * R1 + cos3 * R0) >> 16); // (-3.28) + (-3.28) = 4.28 ==> 4.12
        out2 = (int16_t)((-sin3 * R0 + cos3 * R1) >> 16); // (-3.28) + (-3.28) = 4.28 ==> 4.12
        /* writing output(xd', yd') in little endian format */
        data[i3 * 2] = out1;  // 4.12
        data[(i3 * 2) + 1] = out2;  // 4.12

        /* Twiddle coefficients index modifier */
        ic = ic + modifier;

        /* Updating input index */
        i0 = i0 + 1;

    } while(--j);
    // data is in 4.12(q12) format
    // end of first stage processing


    // Start of middle stage processing

    // Twiddle coefficients index modifier
    modifier <<= 2;

    // Middle stage processing...
    for (k = len / 4; k > 4; k >>= 2)
    {
        /* Initializations for the middle stage */
        n1 = n2;
        n2 >>= 2;
        ic = 0;

        for (j = 0; j <= (n2 - 1); j++)
        {
            /* index calculation for the coefficients */
            cos1 = twiddles[ic * 2]; // 1.15
            sin1 = twiddles[(ic * 2) + 1];
            cos2 = twiddles[2 * (ic * 2)];
            sin2 = twiddles[(2 * (ic * 2)) + 1];
            cos3 = twiddles[3 * (ic * 2)];
            sin3 = twiddles[(3 * (ic * 2)) + 1];

            /* Twiddle coefficients index modifier */
            ic = ic + modifier;

            /* Butterfly implementation */
            for (i0 = j; i0 < len; i0 += n1)
            {
                /* index calculation for the inputs... */
                /* data[i0 + 0], pSrc16[i0 + len/4], pSrc16[i0 + fftLen/2], pSrc16[i0 + 3fftLen/4] */
                i1 = i0 + n2;
                i2 = i1 + n2;
                i3 = i2 + n2;

                /* Reading i0, i0+len/2 inputs */
                /* Read ya(real), xa(imag) input */
                T0 = data[i0 * 2];  // 4.12
                T1 = data[(i0 * 2) + 1];  // 4.12

                /* Read yc(real), xc(imag) input */
                S0 = data[i2 * 2];  // 4.12
                S1 = data[(i2 * 2) + 1];  // 4.12

                /* R0 = (ya + yc), R1 = (xa + xc) */
                R0 = __SSAT_16((int32_t)T0 + S0);  // 4.12
                R1 = __SSAT_16((int32_t)T1 + S1);  // 4.12

                /* S0 = (ya - yc), S1 =(xa - xc) */
                S0 = __SSAT_16((int32_t)T0 - S0);  // 4.12
                S1 = __SSAT_16((int32_t)T1 - S1);  // 4.12

                /* Reading i0+len/4, i0+3fftLen/4 inputs */
                /* Read yb(real), xb(imag) input */
                T0 = data[i1 * 2];  // 4.12
                T1 = data[(i1 * 2) + 1];  // 4.12

                /* Read yd(real), xd(imag) input */
                U0 = data[i3 * 2];  // 4.12
                U1 = data[(i3 * 2) + 1];  // 4.12

                /* T0 = (yb + yd), T1 = (xb + xd) */
                T0 = __SSAT_16((int32_t)T0 + U0);  // 4.12
                T1 = __SSAT_16((int32_t)T1 + U1);  // 4.12

                /* writing the butterfly processed i0 sample */

                /* xa' = xa + xb + xc + xd */
                /* ya' = ya + yb + yc + yd */
                out1 = ((R0 >> 1) + (T0 >> 1)) >> 1;  // 6.10
                out2 = ((R1 >> 1) + (T1 >> 1)) >> 1;  // 6.10

                data[i0 * 2] = out1;  // 6.10
                data[(i0 * 2) + 1] = out2;  // 6.10

                /* R0 = (ya + yc) - (yb + yd), R1 = (xa + xc) - (xb + xd) */
                R0 = (R0 >> 1) - (T0 >> 1);  // 5.11
                R1 = (R1 >> 1) - (T1 >> 1);  // 5.11

                /* (ya-yb+yc-yd) * sin2 + (xa-xb+xc-xd) * cos2 */
                /* (ya-yb+yc-yd) * cos2 - (xa-xb+xc-xd) * sin2 */
                out1 = (int16_t)((cos2 * R0 + sin2 * R1) >> 16);  // (-5.26) + (-5.26) = 6.26 ==> 6.10
                out2 = (int16_t)((-sin2 * R0 + cos2 * R1) >> 16);  // (-5.26) + (-5.26) = 6.26 ==> 6.10

                /* Reading i0+3len/4 */
                /* Read yb(real), xb(imag) input */
                T0 = data[i1 * 2];  // 4.12
                T1 = data[(i1 * 2) + 1];  // 4.12

                /* writing the butterfly processed i0 + len/4 sample */
                /* xc' = (xa-xb+xc-xd) * cos2 + (ya-yb+yc-yd) * sin2 */
                /* yc' = (ya-yb+yc-yd) * cos2 - (xa-xb+xc-xd) * sin2 */
                data[i1 * 2] = out1;  // 6.10
                data[(i1 * 2) + 1] = out2;  // 6.10

                /* Butterfly calculations */

                /* Read yd(real), xd(imag) input */
                U0 = data[i3 * 2];  // 4.12
                U1 = data[(i3 * 2) + 1];  // 4.12

                /* T0 = yb-yd, T1 = xb-xd */
                T0 = __SSAT_16((int32_t)T0 - U0);  // 4.12
                T1 = __SSAT_16((int32_t)T1 - U1);  // 4.12

                /* R0 = (ya-yc) + (xb- xd), R1 = (xa-xc) - (yb-yd) */
                R0 = (S0 >> 1) - (T1 >> 1);  // 5.11
                R1 = (S1 >> 1) + (T0 >> 1);  // 5.11

                /* S0 = (ya-yc) - (xb- xd), S1 = (xa-xc) + (yb-yd) */
                S0 = (S0 >> 1) + (T1 >> 1);  // 5.11
                S1 = (S1 >> 1) - (T0 >> 1);  // 5.11

                /* Butterfly process for the i0+len/2 sample */
                out1 = (int16_t)((cos1 * S0 + sin1 * S1) >> 16);  // (-5.26) + (-5.26) = 6.26 ==> 6.10
                out2 = (int16_t)((-sin1 * S0 + cos1 * S1) >> 16);  // (-5.26) + (-5.26) = 6.26 ==> 6.10

                /* xb' = (xa+yb-xc-yd) * cos1 + (ya-xb-yc+xd) * sin1 */
                /* yb' = (ya-xb-yc+xd) * cos1 - (xa+yb-xc-yd) * sin1 */
                data[i2 * 2] = out1;  // 6.10
                data[(i2 * 2) + 1] = out2;  // 6.10

                /* Butterfly process for the i0+3len/4 sample */
                out1 = (int16_t)((sin3 * R1 + cos3 * R0) >> 16);  // (-5.26) + (-5.26) = 6.26 ==> 6.10
                out2 = (int16_t)((-sin3 * R0 + cos3 * R1) >> 16);  // (-5.26) + (-5.26) = 6.26 ==> 6.10

                /* xd' = (xa-yb-xc+yd) * cos3 + (ya+xb-yc-xd) * sin3 */
                /* yd' = (ya+xb-yc-xd) * cos3 - (xa-yb-xc+yd) * sin3 */
                data[i3 * 2] = out1;  // 6.10
                data[(i3 * 2) + 1] = out2;  // 6.10
            }
        }
        /* Twiddle coefficients index modifier */
        modifier <<= 2;
    }
    // end of middle stage processing


    // data is in 10.6(q6) format for the 1024 point
    // data is in 8.8(q8) format for the 256 point
    // data is in 6.10(q10) format for the 64 point
    // data is in 4.12(q12) format for the 16 point

    // Initialization for the last stage
    n1 = n2;
    n2 >>= 2;

    // Last stage processing...

    /* Butterfly implementation */
    for (i0 = 0; i0 <= (len - n1); i0 += n1)
    {
        /* index calculation for the inputs... */
        /* data[i0 + 0], pSrc16[i0 + len/4], pSrc16[i0 + fftLen/2], pSrc16[i0 + 3fftLen/4] */
        i1 = i0 + n2;
        i2 = i1 + n2;
        i3 = i2 + n2;

        /* Reading i0, i0+len/2 inputs */
        /* Read ya (real), xa(imag) input */
        T0 = data[i0 * 2];  // 8.8
        T1 = data[(i0 * 2) + 1];  // 8.8

        /* Read yc(real), xc(imag) input */
        S0 = data[i2 * 2];  // 8.8
        S1 = data[(i2 * 2) + 1];  // 8.8

        /* R0 = (ya + yc), R1 = (xa + xc) */
        R0 = __SSAT_16((int32_t)T0 + S0);  // 8.8
        R1 = __SSAT_16((int32_t)T1 + S1);  // 8.8

        /* S0 = (ya - yc), S1 = (xa - xc) */
        S0 = __SSAT_16((int32_t)T0 - S0);  // 8.8
        S1 = __SSAT_16((int32_t)T1 - S1);  // 8.8

        /* Reading i0+len/4, i0+3fftLen/4 inputs */
        /* Read yb (real), xb(imag) input */
        T0 = data[i1 * 2];  // 8.8
        T1 = data[(i1 * 2) + 1];  // 8.8

        /* Read yd (real), xd(imag) input */
        U0 = data[i3 * 2];  // 8.8
        U1 = data[(i3 * 2) + 1];  // 8.8

        /* T0 = (yb + yd), T1 = (xb + xd) */
        T0 = __SSAT_16((int32_t)T0 + U0);  // 8.8
        T1 = __SSAT_16((int32_t)T1 + U1);  // 8.8

        /* writing the butterfly processed i0 sample */
        /* xa' = (xa+xb+xc+xd) */
        /* ya' = (ya+yb+yc+yd) */
        data[i0 * 2] = (R0 >> 1) + (T0 >> 1);  // 9.7
        data[(i0 * 2) + 1] = (R1 >> 1) + (T1 >> 1);  // 9.7

        /* R0 = (ya + yc) - (yb + yd), R1 = (xa + xc) - (xb + xd) */
        R0 = (R0 >> 1) - (T0 >> 1);  // 9.7
        R1 = (R1 >> 1) - (T1 >> 1);  // 9.7

        /* Read yb(real), xb(imag) input */
        T0 = data[i1 * 2];  // 8.8
        T1 = data[(i1 * 2) + 1];  // 8.8

        /* writing the butterfly processed i0 + len/4 sample */
        /* xc' = (xa-xb+xc-xd) */
        /* yc' = (ya-yb+yc-yd) */
        data[i1 * 2] = R0;  // 9.7
        data[(i1 * 2) + 1] = R1;  // 9.7

        /* Read yd(real), xd(imag) input */
        U0 = data[i3 * 2];  // 8.8
        U1 = data[(i3 * 2) + 1];  // 8.8

        /* T0 = (yb - yd), T1 = (xb - xd)  */
        T0 = __SSAT_16((int32_t)T0 - U0);  // 8.8
        T1 = __SSAT_16((int32_t)T1 - U1);  // 8.8

        /* writing the butterfly processed i0 + len/2 sample */
        /* xb' = (xa+yb-xc-yd) */
        /* yb' = (ya-xb-yc+xd) */
        data[i2 * 2] = (S0 >> 1) + (T1 >> 1);  // 9.7
        data[(i2 * 2) + 1] = (S1 >> 1) - (T0 >> 1);  // 9.7

        /* writing the butterfly processed i0 + 3len/4 sample */
        /* xd' = (xa-yb-xc+yd) */
        /* yd' = (ya+xb-yc-xd) */
        data[i3 * 2] = (S0 >> 1) - (T1 >> 1);  // 9.7
        data[(i3 * 2) + 1] = (S1 >> 1) + (T0 >> 1);  // 9.7
    }
    // end of last stage processing
    // output is in 11.5(q5) format for the 1024 point
    // output is in 9.7(q7) format for the 256 point
    // output is in 7.9(q9) format for the 64 point
    // output is in 5.11(q11) format for the 16 point
}

/*!
 *****************************************************************************
 * \brief Core Real-FFT process
 * Computes the first N/2 bins of N-bin FFT using the N/2 bin result of complex-FFT.
 * Ar=[0..1], Ai=[-0.5..0.5], Br=[0..1], Bi=[-0.5..0.5]
 *
 * \param *pSrc - array of N/2 complex-values (input)
 * \param len - length of the FFT (this is N/2).
 * \param *pATable - A twiddle coefficient buffer.
 * \param *pBTable - B twiddle coefficient buffer.
 * \param modifier - A/B twiddle coefficient modifier that supports different size FFTs with the same twiddle factor tables.
 *****************************************************************************
 */
void split_rfft(int16_t *data, uint16_t len)
{   //Perform a real-only fft split after a regular FFT butterfly
    uint16_t i;
    int32_t real, imag;
    int16_t start_r, start_i, end_r, end_i;
    int16_t twiddleA_r, twiddleA_i, twiddleB_r, twiddleB_i;

    for (i=0; i <= (len / 2); i++)
    {
        start_r = data[i * 2];
        start_i = data[i * 2 + 1];
        end_r = data[(len - i) * 2];
        end_i = data[(len - i) * 2 + 1];
        //Do the first half
        twiddleA_r = twiddlesA[i * MODIFIER(len) * 2];
        twiddleA_i = twiddlesA[i * MODIFIER(len) * 2 + 1];
        twiddleB_r = twiddlesB[i * MODIFIER(len) * 2];
        twiddleB_i = twiddlesB[i * MODIFIER(len) * 2 + 1];
        real = (start_r * twiddleA_r) - (start_i * twiddleA_i)
                + (end_r * twiddleB_r) + (end_i * twiddleB_i);
        imag = (start_i * twiddleA_r) + (start_r * twiddleA_i)
                + (end_r * twiddleB_i) - (end_i * twiddleB_r);
        //Write in place
        data[i * 2] = (int16_t)(real >> 15);
        data[i * 2 + 1] = (int16_t)(imag >> 15);
        if (i < (len / 2)) { //N/4 = N/2 - N/4
            //Do the last half
            twiddleA_r = twiddlesA[(len - i) * MODIFIER(len) * 2];
            twiddleA_i = twiddlesA[(len - i) * MODIFIER(len) * 2 + 1];
            twiddleB_r = twiddlesB[(len - i) * MODIFIER(len) * 2];
            twiddleB_i = twiddlesB[(len - i) * MODIFIER(len) * 2 + 1];
            real = (end_r * twiddleA_r) - (end_i * twiddleA_i)
                + (start_r * twiddleB_r) + (start_i * twiddleB_i);
            imag = (end_i * twiddleA_r) + (end_r * twiddleA_i)
                + (start_r * twiddleB_i) - (start_i * twiddleB_r);
            //Write in place
            data[(len - i) * 2] = (int16_t)(real >> 15);
            data[(len - i) * 2 + 1] = (int16_t)(imag >> 15);
        }
    }
}

static uint32_t ams_isqrt(uint32_t x)
{
    register uint32_t result, tmp;

    result = 0;
    tmp = (1 << 30);  // second-to-top bit set
    while (tmp > x) {
        tmp >>= 2;
    }
    while (tmp != 0) {
        if (x >= (result + tmp)) {
            x -= result + tmp;
            result += 2 * tmp;  // <-- faster than 2 * one
        }
        result >>= 1;
        tmp >>= 2;
    }
    return result;
}

static void normalize_data(int16_t *data, uint16_t size)
{
    int i;
    int min, max, shift;

    min = data[0];
    max = data[0];
    for (i = 0; i < size; ++i) {
        if (data[i] > max) {
            max = data[i];
        } else if (data[i] < min) {
            min = data[i];
        }
    }
    min += (max - min) / 2;
    max -= min;
    shift = 0;
    //@TODO ?
    for (i = 0; i < 15; ++i) {
        if (((1 << (i + 1)) * max) > 0x7FFF) {
            shift = i;
            break;
        }
    }
    for (i = 0; i < size; ++i) {
        data[i] = (data[i] - min) << shift;
    }
}

int ams_rfft(int16_t *data, enum fft_size size)
{   //Assumes only real data
    if (size > (MAX_FFT_LEN * 2)) {
        //@TODO add return codes so we know it failed
        return 0;
    }
    normalize_data(data, size); //@TODO this should be a separate call
    radix4_butterfly(data, size / 2);
    bit_reverse(data, size / 2);
    split_rfft(data, size / 2);

    return 1;
}

void ams_get_magnitude(int16_t *data, uint16_t *buffer, uint16_t size)
{   //data must be twice as long as size
    int i;
    int32_t square;

    for (i = 0; i < size; ++i) { 
        //sqrt(real^2 + imaginary^2)
        square = (data[i * 2] * data[i * 2]) + (data[i * 2 + 1] * data[i * 2 + 1]);
        buffer[i] = ams_isqrt(square);
    }
}

